import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level, min_sdk_level
import objc


class TestNSDraggingHelper(AppKit.NSObject):
    def draggingFormation(self):
        return 1

    def setDraggingFormation_(self, v):
        return 1

    def animatesToDestination(self):
        return 1

    def setAnimatesToDestination_(self, v):
        return 1

    def numberOfValidItemsForDrop(self):
        return 1

    def setNumberOfValidItemsForDrop_(self, v):
        return 1

    def enumerateDraggingItemsWithOptions_forView_classes_searchOptions_usingBlock_(
        self, a, b, c, d, e
    ):
        return 1

    def draggingSession_sourceOperationMaskForDraggingContext_(self, a, b):
        return 1

    def draggingSession_willBeginAtPoint_(self, a, b):
        pass

    def draggingSession_movedToPoint_(self, a, b):
        pass

    def draggingSession_endedAtPoint_operation_(self, a, b, c):
        pass

    def ignoreModifierKeysForDraggingSession_(self, a):
        return 1

    def draggingSourceOperationMask(self):
        return 1

    def draggingLocation(self):
        return 1

    def draggedImageLocation(self):
        return 1

    def draggingSource(self):
        return 1

    def draggingSequenceNumber(self):
        return 1

    def slideDraggedImageTo_(self, v):
        pass

    def draggingEntered_(self, sender):
        return 1

    def draggingUpdated_(self, sender):
        return 1

    def draggingExited_(self, sender):
        pass

    def prepareForDragOperation_(self, sender):
        return 1

    def performDragOperation_(self, sender):
        return 1

    def concludeDragOperation_(self, sender):
        pass

    def draggingEnded_(self, sender):
        pass

    def wantsPeriodicDraggingUpdates(self):
        return 1

    def draggingSourceOperationMaskForLocal_(self, v):
        return 1

    def draggedImage_beganAt_(self, v, v2):
        pass

    def draggedImage_endedAt_operation_(self, v, v2, v3):
        pass

    def draggedImage_movedTo_(self, v, v2):
        pass

    def ignoreModifierKeysWhileDragging(self):
        return 1

    def draggedImage_endedAt_deposited_(self, i, p, f):
        pass

    def springLoadingActivated_draggingInfo_(self, a, i):
        pass

    def springLoadingEntered_(self, a):
        pass

    def springLoadingUpdated_(self, a):
        pass


class TestNSDragging(TestCase):
    def testConstants(self):
        self.assertEqual(AppKit.NSDragOperationNone, 0)
        self.assertEqual(AppKit.NSDragOperationCopy, 1)
        self.assertEqual(AppKit.NSDragOperationLink, 2)
        self.assertEqual(AppKit.NSDragOperationGeneric, 4)
        self.assertEqual(AppKit.NSDragOperationPrivate, 8)
        self.assertEqual(AppKit.NSDragOperationAll_Obsolete, 15)
        self.assertEqual(AppKit.NSDragOperationMove, 16)
        self.assertEqual(AppKit.NSDragOperationDelete, 32)
        self.assertEqual(AppKit.NSDragOperationEvery, AppKit.NSUIntegerMax)

        self.assertEqual(AppKit.NSDragOperationAll, AppKit.NSDragOperationAll_Obsolete)

    @min_os_level("10.7")
    def testConstant10_7(self):
        self.assertEqual(AppKit.NSDraggingFormationDefault, 0)
        self.assertEqual(AppKit.NSDraggingFormationNone, 1)
        self.assertEqual(AppKit.NSDraggingFormationPile, 2)
        self.assertEqual(AppKit.NSDraggingFormationList, 3)
        self.assertEqual(AppKit.NSDraggingFormationStack, 4)

        self.assertEqual(AppKit.NSDraggingContextOutsideApplication, 0)
        self.assertEqual(AppKit.NSDraggingContextWithinApplication, 1)

        self.assertEqual(
            AppKit.NSDraggingItemEnumerationConcurrent, AppKit.NSEnumerationConcurrent
        )
        self.assertEqual(
            AppKit.NSDraggingItemEnumerationClearNonenumeratedImages, 1 << 16
        )

    @min_os_level("10.11")
    def testConstant10_11(self):
        self.assertEqual(AppKit.NSSpringLoadingHighlightNone, 0)
        self.assertEqual(AppKit.NSSpringLoadingHighlightStandard, 1)
        self.assertEqual(AppKit.NSSpringLoadingHighlightEmphasized, 2)

        self.assertEqual(AppKit.NSSpringLoadingDisabled, 0)
        self.assertEqual(AppKit.NSSpringLoadingEnabled, 1)
        self.assertEqual(AppKit.NSSpringLoadingContinuousActivation, 2)
        self.assertEqual(AppKit.NSSpringLoadingNoHover, 4)

    @min_sdk_level("10.7")
    def testProtocols(self):
        objc.protocolNamed("NSDraggingDestination")
        objc.protocolNamed("NSDraggingSource")

    @min_sdk_level("10.11")
    def testProtocols10_11(self):
        objc.protocolNamed("NSSpringLoadingDestination")

    def testProtocolImplementations(self):
        self.assertResultHasType(
            TestNSDraggingHelper.draggingSourceOperationMask, objc._C_NSUInteger
        )
        self.assertResultHasType(
            TestNSDraggingHelper.draggingLocation, AppKit.NSPoint.__typestr__
        )
        self.assertResultHasType(
            TestNSDraggingHelper.draggedImageLocation, AppKit.NSPoint.__typestr__
        )
        self.assertResultHasType(
            TestNSDraggingHelper.draggingSequenceNumber, objc._C_NSInteger
        )
        self.assertArgHasType(
            TestNSDraggingHelper.slideDraggedImageTo_, 0, AppKit.NSPoint.__typestr__
        )

        self.assertResultHasType(
            TestNSDraggingHelper.draggingEntered_, objc._C_NSUInteger
        )
        self.assertResultHasType(
            TestNSDraggingHelper.draggingUpdated_, objc._C_NSUInteger
        )
        self.assertResultIsBOOL(TestNSDraggingHelper.prepareForDragOperation_)
        self.assertResultIsBOOL(TestNSDraggingHelper.performDragOperation_)
        self.assertResultIsBOOL(TestNSDraggingHelper.wantsPeriodicDraggingUpdates)

        self.assertResultHasType(
            TestNSDraggingHelper.draggingSourceOperationMaskForLocal_,
            objc._C_NSUInteger,
        )
        self.assertArgIsBOOL(
            TestNSDraggingHelper.draggingSourceOperationMaskForLocal_, 0
        )
        self.assertArgHasType(
            TestNSDraggingHelper.draggedImage_beganAt_, 1, AppKit.NSPoint.__typestr__
        )
        self.assertArgHasType(
            TestNSDraggingHelper.draggedImage_endedAt_operation_,
            1,
            AppKit.NSPoint.__typestr__,
        )
        self.assertArgHasType(
            TestNSDraggingHelper.draggedImage_endedAt_operation_, 2, objc._C_NSUInteger
        )
        self.assertArgHasType(
            TestNSDraggingHelper.draggedImage_movedTo_, 1, AppKit.NSPoint.__typestr__
        )
        self.assertResultIsBOOL(TestNSDraggingHelper.ignoreModifierKeysWhileDragging)
        self.assertArgHasType(
            TestNSDraggingHelper.draggedImage_endedAt_deposited_,
            1,
            AppKit.NSPoint.__typestr__,
        )
        self.assertArgIsBOOL(TestNSDraggingHelper.draggedImage_endedAt_deposited_, 2)

        self.assertResultHasType(
            TestNSDraggingHelper.draggingSession_sourceOperationMaskForDraggingContext_,
            objc._C_NSUInteger,
        )
        self.assertArgHasType(
            TestNSDraggingHelper.draggingSession_sourceOperationMaskForDraggingContext_,
            1,
            objc._C_NSInteger,
        )

    @min_os_level("10.7")
    def testProtocols10_7(self):
        self.assertResultIsBOOL(TestNSDraggingHelper.animatesToDestination)
        self.assertArgIsBOOL(TestNSDraggingHelper.setAnimatesToDestination_, 0)

        self.assertResultIsBOOL(TestNSDraggingHelper.animatesToDestination)
        self.assertArgIsBOOL(TestNSDraggingHelper.setAnimatesToDestination_, 0)

        self.assertResultHasType(
            TestNSDraggingHelper.numberOfValidItemsForDrop, objc._C_NSInteger
        )
        self.assertArgHasType(
            TestNSDraggingHelper.setNumberOfValidItemsForDrop_, 0, objc._C_NSInteger
        )

        self.assertArgHasType(
            TestNSDraggingHelper.enumerateDraggingItemsWithOptions_forView_classes_searchOptions_usingBlock_,  # noqa: B950
            0,
            objc._C_NSUInteger,
        )
        self.assertArgIsBlock(
            TestNSDraggingHelper.enumerateDraggingItemsWithOptions_forView_classes_searchOptions_usingBlock_,  # noqa: B950
            4,
            b"v@" + objc._C_NSInteger + b"o^" + objc._C_NSBOOL,
        )

        self.assertArgHasType(
            TestNSDraggingHelper.draggingSession_willBeginAtPoint_,
            1,
            AppKit.NSPoint.__typestr__,
        )
        self.assertArgHasType(
            TestNSDraggingHelper.draggingSession_movedToPoint_,
            1,
            AppKit.NSPoint.__typestr__,
        )
        self.assertArgHasType(
            TestNSDraggingHelper.draggingSession_endedAtPoint_operation_,
            1,
            AppKit.NSPoint.__typestr__,
        )
        self.assertArgHasType(
            TestNSDraggingHelper.draggingSession_endedAtPoint_operation_,
            2,
            objc._C_NSUInteger,
        )

        self.assertResultIsBOOL(
            TestNSDraggingHelper.ignoreModifierKeysForDraggingSession_
        )

    @min_os_level("10.11")
    def testProtocolMethods10_11(self):
        self.assertArgIsBOOL(
            TestNSDraggingHelper.springLoadingActivated_draggingInfo_, 0
        )
        self.assertResultHasType(
            TestNSDraggingHelper.springLoadingEntered_, objc._C_NSUInteger
        )
        self.assertResultHasType(
            TestNSDraggingHelper.springLoadingUpdated_, objc._C_NSUInteger
        )



import Cocoa
import objc


class MyBaseGradientView(Cocoa.NSView):
    myGradient = objc.ivar()
    myStartColor = objc.ivar()
    myEndColor = objc.ivar()

    forceColorChange = objc.ivar.bool()
    myAngle = objc.ivar.double()
    myIsRadial = objc.ivar.bool()
    myOffsetPt = objc.ivar.NSPoint()

    def resetGradient(self):
        if self.forceColorChange and self.myGradient is not None:
            self.myGradient = None

        if self.myGradient is None:
            self.myGradient = (
                Cocoa.NSGradient.alloc().initWithStartingColor_endingColor_(
                    self.myStartColor, self.myEndColor
                )
            )
            self.forceColorChange = False

    def setStartColor_(self, startColor):
        self.myStartColor = startColor
        self.forceColorChange = True
        self.setNeedsDisplay_(True)

    def setEndColor_(self, endColor):
        self.myEndColor = endColor
        self.forceColorChange = True
        self.setNeedsDisplay_(True)

    def setAngle_(self, angle):
        self.myAngle = angle
        self.setNeedsDisplay_(True)

    def setRadialDraw_(self, isRadial):
        self.myIsRadial = isRadial
        self.setNeedsDisplay_(True)

    def getRelativeCenterPositionFromEvent_(self, theEvent):
        curMousePt = self.convertPoint_fromView_(theEvent.locationInWindow(), None)
        pt = Cocoa.NSMakePoint(
            (curMousePt.x - Cocoa.NSMidX(self.bounds()))
            / (self.bounds().size.width / 2.0),
            (curMousePt.y - Cocoa.NSMidY(self.bounds()))
            / (self.bounds().size.height / 2.0),
        )
        return pt

    def mouseDown_(self, theEvent):
        if self.myIsRadial:
            self.myOffsetPt = self.getRelativeCenterPositionFromEvent_(theEvent)
            self.setNeedsDisplay_(True)

    def mouseDragged_(self, theEvent):
        if self.myIsRadial:
            self.myOffsetPt = self.getRelativeCenterPositionFromEvent_(theEvent)
            self.setNeedsDisplay_(True)



"""
Script for building the example.

Usage:
    python3 setup.py py2app
"""
from setuptools import setup

setup(
    app=["PythonBrowser.py"],
    data_files=["MainMenu.nib", "PythonBrowser.nib"],
    setup_requires=["py2app", "pyobjc-framework-Cocoa"],
)



import Foundation
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSRunLoop(TestCase):
    def testMethods(self):
        self.assertResultIsBOOL(Foundation.NSRunLoop.runMode_beforeDate_)

        self.assertArgIsSEL(
            Foundation.NSRunLoop.performSelector_withObject_afterDelay_inModes_,
            0,
            b"v@:@",
        )
        self.assertArgIsSEL(
            Foundation.NSRunLoop.performSelector_withObject_afterDelay_, 0, b"v@:@"
        )
        self.assertArgIsSEL(
            Foundation.NSRunLoop.performSelector_target_argument_order_modes_,
            0,
            b"v@:@",
        )

    @min_os_level("10.12")
    def testMethods10_12(self):
        self.assertArgIsBlock(Foundation.NSRunLoop.performInModes_block_, 1, b"v")
        self.assertArgIsBlock(Foundation.NSRunLoop.performBlock_, 0, b"v")

    def testConstants(self):
        self.assertIsInstance(Foundation.NSDefaultRunLoopMode, str)
        self.assertIsInstance(Foundation.NSRunLoopCommonModes, str)



import Foundation
from PyObjCTools.TestSupport import TestCase, min_sdk_level
import objc


class TestNSConnectionHelper(Foundation.NSObject):
    def makeNewConnection_sender_(self, a, b):
        return 1

    def connection_shouldMakeNewConnection_(self, a, b):
        return 1

    def authenticateComponents_withData_(self, a, b):
        return 1

    def connection_handleRequest_(self, a, b):
        return 1


class TestNSConnection(TestCase):
    def testConstants(self):
        self.assertIsInstance(Foundation.NSConnectionReplyMode, str)
        self.assertIsInstance(Foundation.NSConnectionDidDieNotification, str)
        self.assertIsInstance(Foundation.NSFailedAuthenticationException, str)
        self.assertIsInstance(Foundation.NSConnectionDidInitializeNotification, str)

    def testMethods(self):
        self.assertArgIsBOOL(
            Foundation.NSConnection.setIndependentConversationQueueing_, 0
        )
        self.assertResultIsBOOL(Foundation.NSConnection.independentConversationQueueing)
        self.assertResultIsBOOL(Foundation.NSConnection.isValid)
        self.assertResultIsBOOL(Foundation.NSConnection.registerName_)
        self.assertResultIsBOOL(Foundation.NSConnection.registerName_withNameServer_)
        self.assertResultIsBOOL(Foundation.NSConnection.multipleThreadsEnabled)

    def testProtocols(self):
        self.assertResultIsBOOL(TestNSConnectionHelper.makeNewConnection_sender_)
        self.assertResultIsBOOL(
            TestNSConnectionHelper.connection_shouldMakeNewConnection_
        )
        self.assertResultIsBOOL(TestNSConnectionHelper.authenticateComponents_withData_)
        self.assertResultIsBOOL(TestNSConnectionHelper.connection_handleRequest_)

    @min_sdk_level("10.10")
    def testProtocolObjects(self):
        objc.protocolNamed("NSConnectionDelegate")



import Foundation
from PyObjCTools.TestSupport import TestCase


class TestNSNull(TestCase):
    def testBool(self):
        v = Foundation.NSNull.null()
        self.assertFalse(v)
        self.assertIsNot(v, None)



# Test special methods of AppKit.NSBezierPath
# TODO: Test implementing these in python.
import objc
import AppKit
from PyObjCTools.TestSupport import TestCase


class TestNSBezierPath(TestCase):
    if not hasattr(TestCase, "assertAlmostEquals"):

        def assertAlmostEquals(self, val1, val2):
            self.assertTrue(abs(val1 - val2) < 0.000_001)

    def assertPointEquals(self, point1, point2):
        self.assertAlmostEqual(point1[0], point2[0])
        self.assertAlmostEqual(point1[1], point2[1])

    def test_creation(self):
        p = AppKit.NSBezierPath.bezierPath()
        self.assertIsNot(p, None)
        self.assertEqual(p.elementCount(), 0)

        p = AppKit.NSBezierPath.bezierPathWithOvalInRect_(((0, 0), (100, 50)))
        self.assertIsNot(p, None)
        self.assertEqual(p.elementCount(), 5)

    def test_appendPoints(self):
        p = AppKit.NSBezierPath.bezierPath()
        self.assertIsNot(p, None)
        self.assertEqual(p.elementCount(), 0)

        points = [(0, 0), (100, 0), (100, 100), (0, 0)]
        p.appendBezierPathWithPoints_count_(points, 3)
        self.assertEqual(p.elementCount(), 3)

    def test_setLineDash(self):
        p = AppKit.NSBezierPath.bezierPath()
        p.setLineDash_count_phase_((10, 10, 20, 5), 4, 45.0)

        pattern, count, phase = p.getLineDash_count_phase_(objc.NULL, 0, None)
        # self.assertEqual(pattern, None)
        self.assertEqual(pattern, objc.NULL)
        self.assertEqual(count, 4)
        self.assertAlmostEqual(phase, 45.0)

        pattern, count, phase = p.getLineDash_count_phase_(None, 4, None)
        self.assertAlmostEqual(pattern[0], 10)
        self.assertAlmostEqual(pattern[1], 10)
        self.assertAlmostEqual(pattern[2], 20)
        self.assertAlmostEqual(pattern[3], 5)
        self.assertEqual(count, 4)
        self.assertAlmostEqual(phase, 45.0)

    def test_elementAtIndex(self):
        p = AppKit.NSBezierPath.bezierPath()
        p.moveToPoint_((10, 10))
        p.lineToPoint_((20, 30))
        p.lineToPoint_((30, 20))
        p.curveToPoint_controlPoint1_controlPoint2_((40, 41), (10, 11), (20, 21))
        p.closePath()

        self.assertEqual(p.elementAtIndex_(0), AppKit.NSMoveToBezierPathElement)
        self.assertEqual(p.elementAtIndex_(1), AppKit.NSLineToBezierPathElement)
        self.assertEqual(p.elementAtIndex_(2), AppKit.NSLineToBezierPathElement)
        self.assertEqual(p.elementAtIndex_(3), AppKit.NSCurveToBezierPathElement)
        self.assertEqual(p.elementAtIndex_(4), AppKit.NSClosePathBezierPathElement)

        tp, points = p.elementAtIndex_associatedPoints_(0)
        self.assertEqual(tp, AppKit.NSMoveToBezierPathElement)
        self.assertEqual(len(points), 1)
        self.assertPointEquals(points[0], (10, 10))

        tp, points = p.elementAtIndex_associatedPoints_(1)
        self.assertEqual(tp, AppKit.NSLineToBezierPathElement)
        self.assertEqual(len(points), 1)
        self.assertPointEquals(points[0], (20, 30))

        tp, points = p.elementAtIndex_associatedPoints_(2)
        self.assertEqual(tp, AppKit.NSLineToBezierPathElement)
        self.assertEqual(len(points), 1)
        self.assertPointEquals(points[0], (30, 20))

        tp, points = p.elementAtIndex_associatedPoints_(3)
        self.assertEqual(tp, AppKit.NSCurveToBezierPathElement)
        self.assertEqual(len(points), 3)
        self.assertPointEquals(points[0], (10, 11))  # control point 1
        self.assertPointEquals(points[1], (20, 21))  # control point 2
        self.assertPointEquals(points[2], (40, 41))  # end point

        tp, points = p.elementAtIndex_associatedPoints_(4)
        self.assertEqual(tp, AppKit.NSClosePathBezierPathElement)
        self.assertEqual(len(points), 0)

    def test_setAssociatedPoints(self):
        p = AppKit.NSBezierPath.bezierPath()
        p.moveToPoint_((10, 10))
        p.lineToPoint_((20, 30))
        p.lineToPoint_((30, 20))
        p.curveToPoint_controlPoint1_controlPoint2_((40, 41), (10, 11), (20, 21))
        p.closePath()

        p.setAssociatedPoints_atIndex_([(0, 1)], 0)
        tp, points = p.elementAtIndex_associatedPoints_(0)
        self.assertEqual(tp, AppKit.NSMoveToBezierPathElement)
        self.assertEqual(len(points), 1)
        self.assertPointEquals(points[0], (0, 1))

        p.setAssociatedPoints_atIndex_([(0, 1), (2, 3), (3, 4)], 3)
        tp, points = p.elementAtIndex_associatedPoints_(3)
        self.assertEqual(tp, AppKit.NSCurveToBezierPathElement)
        self.assertEqual(len(points), 3)
        self.assertPointEquals(points[0], (0, 1))  # control point 1
        self.assertPointEquals(points[1], (2, 3))  # control point 2
        self.assertPointEquals(points[2], (3, 4))  # end point

    def testConstants(self):
        self.assertEqual(AppKit.NSButtLineCapStyle, 0)
        self.assertEqual(AppKit.NSRoundLineCapStyle, 1)
        self.assertEqual(AppKit.NSSquareLineCapStyle, 2)

        self.assertEqual(AppKit.NSMiterLineJoinStyle, 0)
        self.assertEqual(AppKit.NSRoundLineJoinStyle, 1)
        self.assertEqual(AppKit.NSBevelLineJoinStyle, 2)

        self.assertEqual(AppKit.NSNonZeroWindingRule, 0)
        self.assertEqual(AppKit.NSEvenOddWindingRule, 1)

        self.assertEqual(AppKit.NSMoveToBezierPathElement, 0)
        self.assertEqual(AppKit.NSLineToBezierPathElement, 1)
        self.assertEqual(AppKit.NSCurveToBezierPathElement, 2)
        self.assertEqual(AppKit.NSClosePathBezierPathElement, 3)

        self.assertEqual(AppKit.NSLineCapStyleButt, 0)
        self.assertEqual(AppKit.NSLineCapStyleRound, 1)
        self.assertEqual(AppKit.NSLineCapStyleSquare, 2)

        self.assertEqual(AppKit.NSLineJoinStyleMiter, 0)
        self.assertEqual(AppKit.NSLineJoinStyleRound, 1)
        self.assertEqual(AppKit.NSLineJoinStyleBevel, 2)

        self.assertEqual(AppKit.NSWindingRuleNonZero, 0)
        self.assertEqual(AppKit.NSWindingRuleEvenOdd, 1)

        self.assertEqual(AppKit.NSBezierPathElementMoveTo, 0)
        self.assertEqual(AppKit.NSBezierPathElementLineTo, 1)
        self.assertEqual(AppKit.NSBezierPathElementCurveTo, 2)
        self.assertEqual(AppKit.NSBezierPathElementClosePath, 3)

    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSBezierPath.isEmpty)
        self.assertResultIsBOOL(AppKit.NSBezierPath.containsPoint_)
        self.assertResultIsBOOL(AppKit.NSBezierPath.cachesBezierPath)
        self.assertArgIsBOOL(AppKit.NSBezierPath.setCachesBezierPath_, 0)
        self.assertArgIsBOOL(
            AppKit.NSBezierPath.appendBezierPathWithArcWithCenter_radius_startAngle_endAngle_clockwise_,  # noqa: B950
            4,
        )

        self.assertArgSizeInArg(
            AppKit.NSBezierPath.appendBezierPathWithGlyphs_count_inFont_, 0, 1
        )

        # XXX: AppKit.NSBezierPath.drawPackedGlyphs_atPoint_
        # XXX: AppKit.NSBezierPath.appendBezierPathWithPackedGlyphs_



import Foundation
from PyObjCTools.TestSupport import TestCase


class TestNSScriptCommand(TestCase):
    def testConstants(self):
        self.assertEqual(Foundation.NSNoScriptError, 0)
        self.assertEqual(Foundation.NSReceiverEvaluationScriptError, 1)
        self.assertEqual(Foundation.NSKeySpecifierEvaluationScriptError, 2)
        self.assertEqual(Foundation.NSArgumentEvaluationScriptError, 3)
        self.assertEqual(Foundation.NSReceiversCantHandleCommandScriptError, 4)
        self.assertEqual(Foundation.NSRequiredArgumentsMissingScriptError, 5)
        self.assertEqual(Foundation.NSArgumentsWrongScriptError, 6)
        self.assertEqual(Foundation.NSUnknownKeyScriptError, 7)
        self.assertEqual(Foundation.NSInternalScriptError, 8)
        self.assertEqual(Foundation.NSOperationNotSupportedForKeyScriptError, 9)
        self.assertEqual(Foundation.NSCannotCreateScriptCommandError, 10)

    def testMethods(self):
        self.assertResultIsBOOL(Foundation.NSScriptCommand.isWellFormed)



import AppKit
from PyObjCTools.TestSupport import TestCase


class TestNSFontManagerHelper(AppKit.NSObject):
    def fontManager_willIncludeFont_(self, m, f):
        return 1


class TestNSFontManager(TestCase):
    def testConstants(self):
        self.assertEqual(AppKit.NSItalicFontMask, 0x00000001)
        self.assertEqual(AppKit.NSBoldFontMask, 0x00000002)
        self.assertEqual(AppKit.NSUnboldFontMask, 0x00000004)
        self.assertEqual(AppKit.NSNonStandardCharacterSetFontMask, 0x00000008)
        self.assertEqual(AppKit.NSNarrowFontMask, 0x00000010)
        self.assertEqual(AppKit.NSExpandedFontMask, 0x00000020)
        self.assertEqual(AppKit.NSCondensedFontMask, 0x00000040)
        self.assertEqual(AppKit.NSSmallCapsFontMask, 0x00000080)
        self.assertEqual(AppKit.NSPosterFontMask, 0x00000100)
        self.assertEqual(AppKit.NSCompressedFontMask, 0x00000200)
        self.assertEqual(AppKit.NSFixedPitchFontMask, 0x00000400)
        self.assertEqual(AppKit.NSUnitalicFontMask, 0x01000000)

        self.assertEqual(AppKit.NSFontCollectionApplicationOnlyMask, 1 << 0)

        self.assertEqual(AppKit.NSNoFontChangeAction, 0)
        self.assertEqual(AppKit.NSViaPanelFontAction, 1)
        self.assertEqual(AppKit.NSAddTraitFontAction, 2)
        self.assertEqual(AppKit.NSSizeUpFontAction, 3)
        self.assertEqual(AppKit.NSSizeDownFontAction, 4)
        self.assertEqual(AppKit.NSHeavierFontAction, 5)
        self.assertEqual(AppKit.NSLighterFontAction, 6)
        self.assertEqual(AppKit.NSRemoveTraitFontAction, 7)

    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSFontManager.isMultiple)
        self.assertArgIsBOOL(AppKit.NSFontManager.setSelectedFont_isMultiple_, 1)
        self.assertArgIsBOOL(AppKit.NSFontManager.fontMenu_, 0)
        self.assertArgIsBOOL(AppKit.NSFontManager.fontPanel_, 0)
        self.assertResultIsBOOL(AppKit.NSFontManager.isEnabled)
        self.assertArgIsBOOL(AppKit.NSFontManager.setEnabled_, 0)
        self.assertResultIsBOOL(AppKit.NSFontManager.sendAction)
        self.assertArgIsBOOL(AppKit.NSFontManager.setSelectedAttributes_isMultiple_, 1)
        self.assertResultIsBOOL(AppKit.NSFontManager.addCollection_options_)
        self.assertResultIsBOOL(AppKit.NSFontManager.removeCollection_)
        self.assertResultIsBOOL(AppKit.NSFontManager.fontNamed_hasTraits_)

        self.assertArgIsBOOL(AppKit.NSFontManager.convertWeight_ofFont_, 0)

    def testProtocols(self):
        self.assertResultIsBOOL(TestNSFontManagerHelper.fontManager_willIncludeFont_)



import Foundation
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSURLCredentialStorage(TestCase):
    def testConstants(self):
        self.assertIsInstance(Foundation.NSURLCredentialStorageChangedNotification, str)

    @min_os_level("10.9")
    def testConstants10_9(self):
        self.assertIsInstance(
            Foundation.NSURLCredentialStorageRemoveSynchronizableCredentials, str
        )

    @min_os_level("10.10")
    def testMethods10_10(self):
        self.assertArgIsBlock(
            Foundation.NSURLCredentialStorage.getCredentialsForProtectionSpace_task_completionHandler_,  # noqa: B950
            2,
            b"v@",
        )
        self.assertArgIsBlock(
            Foundation.NSURLCredentialStorage.getDefaultCredentialForProtectionSpace_task_completionHandler_,  # noqa: B950
            2,
            b"v@",
        )



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSFontCollection(TestCase):
    @min_os_level("10.7")
    def testConstants10_7(self):
        self.assertEqual(AppKit.NSFontCollectionVisibilityProcess, 1)
        self.assertEqual(AppKit.NSFontCollectionVisibilityUser, 2)
        self.assertEqual(AppKit.NSFontCollectionVisibilityComputer, 4)

        self.assertIsInstance(AppKit.NSFontCollectionIncludeDisabledFontsOption, str)
        self.assertIsInstance(AppKit.NSFontCollectionRemoveDuplicatesOption, str)
        self.assertIsInstance(AppKit.NSFontCollectionDisallowAutoActivationOption, str)
        self.assertIsInstance(AppKit.NSFontCollectionDidChangeNotification, str)
        self.assertIsInstance(AppKit.NSFontCollectionActionKey, str)
        self.assertIsInstance(AppKit.NSFontCollectionNameKey, str)
        self.assertIsInstance(AppKit.NSFontCollectionOldNameKey, str)
        self.assertIsInstance(AppKit.NSFontCollectionVisibilityKey, str)
        self.assertIsInstance(AppKit.NSFontCollectionWasShown, str)
        self.assertIsInstance(AppKit.NSFontCollectionWasHidden, str)
        self.assertIsInstance(AppKit.NSFontCollectionWasRenamed, str)
        self.assertIsInstance(AppKit.NSFontCollectionAllFonts, str)
        self.assertIsInstance(AppKit.NSFontCollectionUser, str)
        self.assertIsInstance(AppKit.NSFontCollectionFavorites, str)
        self.assertIsInstance(AppKit.NSFontCollectionRecentlyUsed, str)

    @min_os_level("10.7")
    def testMethods10_7(self):
        self.assertResultIsBOOL(
            AppKit.NSFontCollection.showFontCollection_withName_visibility_error_
        )
        self.assertArgIsOut(
            AppKit.NSFontCollection.showFontCollection_withName_visibility_error_, 3
        )
        self.assertResultIsBOOL(
            AppKit.NSFontCollection.hideFontCollectionWithName_visibility_error_
        )
        self.assertArgIsOut(
            AppKit.NSFontCollection.hideFontCollectionWithName_visibility_error_, 2
        )

        self.assertResultIsBOOL(
            AppKit.NSFontCollection.renameFontCollectionWithName_visibility_toName_error_
        )
        self.assertArgIsOut(
            AppKit.NSFontCollection.renameFontCollectionWithName_visibility_toName_error_,
            3,
        )



import Foundation
from PyObjCTools.TestSupport import TestCase


class TestNSValue(TestCase):
    def testMethods(self):
        self.assertArgIsBOOL(Foundation.NSNumber.numberWithBool_, 0)
        self.assertArgIsBOOL(Foundation.NSNumber.initWithBool_, 0)
        self.assertResultIsBOOL(Foundation.NSNumber.boolValue)
        self.assertResultIsBOOL(Foundation.NSNumber.isEqualToNumber_)

        self.assertResultIsBOOL(Foundation.NSValue.isEqualToValue_)

        self.assertArgIsIn(Foundation.NSValue.initWithBytes_objCType_, 0)
        self.assertArgIsVariableSize(Foundation.NSValue.initWithBytes_objCType_, 0)
        self.assertArgHasType(Foundation.NSValue.initWithBytes_objCType_, 1, b"n^t")
        self.assertArgIsNullTerminated(Foundation.NSValue.initWithBytes_objCType_, 1)

        self.assertArgIsIn(Foundation.NSValue.valueWithBytes_objCType_, 0)
        self.assertArgIsVariableSize(Foundation.NSValue.valueWithBytes_objCType_, 0)
        self.assertArgHasType(Foundation.NSValue.valueWithBytes_objCType_, 1, b"n^t")
        self.assertArgIsNullTerminated(Foundation.NSValue.valueWithBytes_objCType_, 1)

        self.assertArgIsIn(Foundation.NSValue.value_withObjCType_, 0)
        self.assertArgIsVariableSize(Foundation.NSValue.value_withObjCType_, 0)
        self.assertArgHasType(Foundation.NSValue.value_withObjCType_, 1, b"n^t")
        self.assertArgIsNullTerminated(Foundation.NSValue.value_withObjCType_, 1)

        self.assertResultIsNullTerminated(Foundation.NSValue.objCType)
        self.assertResultHasType(Foundation.NSValue.objCType, b"^t")



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSTextContainer(TestCase):
    def testConstants(self):
        self.assertEqual(AppKit.NSLineSweepLeft, 0)
        self.assertEqual(AppKit.NSLineSweepRight, 1)
        self.assertEqual(AppKit.NSLineSweepDown, 2)
        self.assertEqual(AppKit.NSLineSweepUp, 3)

        self.assertEqual(AppKit.NSLineDoesntMove, 0)
        self.assertEqual(AppKit.NSLineMovesLeft, 1)
        self.assertEqual(AppKit.NSLineMovesRight, 2)
        self.assertEqual(AppKit.NSLineMovesDown, 3)
        self.assertEqual(AppKit.NSLineMovesUp, 4)

    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSTextContainer.widthTracksTextView)
        self.assertArgIsBOOL(AppKit.NSTextContainer.setWidthTracksTextView_, 0)
        self.assertResultIsBOOL(AppKit.NSTextContainer.heightTracksTextView)
        self.assertArgIsBOOL(AppKit.NSTextContainer.setHeightTracksTextView_, 0)
        self.assertResultIsBOOL(AppKit.NSTextContainer.isSimpleRectangularTextContainer)
        self.assertResultIsBOOL(AppKit.NSTextContainer.containsPoint_)

    @min_os_level("10.11")
    def testMethods10_11(self):
        self.assertArgIsOut(
            AppKit.NSTextContainer.lineFragmentRectForProposedRect_atIndex_writingDirection_remainingRect_,  # noqa: B950
            3,
        )



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSColorSampler(TestCase):
    @min_os_level("10.15")
    def test_methods_10_15(self):
        self.assertArgIsBlock(
            AppKit.NSColorSampler.showSamplerWithSelectionHandler_, 0, b"v@"
        )



#
#  __main__.py
#  TableModel
#
#  Created by Bob Ippolito on Sun Apr 04 2004.
#  Copyright (c) 2004 Bob Ippolito. All rights reserved.
#

# import classes required to start application
import TableModelAppDelegate  # noqa: F401
from PyObjCTools import AppHelper

# start the event loop
AppHelper.runEventLoop(argv=[])



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level, min_sdk_level
import objc


class TestNSTextHelper(AppKit.NSObject):
    def textShouldBeginEditing_(self, t):
        return 1

    def textShouldEndEditing_(self, t):
        return 1


class TestNSText(TestCase):
    def testConstants(self):
        self.assertEqual(AppKit.NSEnterCharacter, chr(0x0003))
        self.assertEqual(AppKit.NSBackspaceCharacter, chr(0x0008))
        self.assertEqual(AppKit.NSTabCharacter, chr(0x0009))
        self.assertEqual(AppKit.NSNewlineCharacter, chr(0x000A))
        self.assertEqual(AppKit.NSFormFeedCharacter, chr(0x000C))
        self.assertEqual(AppKit.NSCarriageReturnCharacter, chr(0x000D))
        self.assertEqual(AppKit.NSBackTabCharacter, chr(0x0019))
        self.assertEqual(AppKit.NSDeleteCharacter, chr(0x007F))
        self.assertEqual(AppKit.NSLineSeparatorCharacter, chr(0x2028))
        self.assertEqual(AppKit.NSParagraphSeparatorCharacter, chr(0x2029))

        self.assertEqual(AppKit.NSLeftTextAlignment, 0)
        self.assertEqual(AppKit.NSRightTextAlignment, 1)
        self.assertEqual(AppKit.NSCenterTextAlignment, 2)
        self.assertEqual(AppKit.NSJustifiedTextAlignment, 3)
        self.assertEqual(AppKit.NSNaturalTextAlignment, 4)

        self.assertEqual(AppKit.NSTextAlignmentLeft, 0)
        if objc.arch == "x86_64":
            self.assertEqual(AppKit.NSTextAlignmentRight, 1)
            self.assertEqual(AppKit.NSTextAlignmentCenter, 2)
        else:
            self.assertEqual(AppKit.NSTextAlignmentCenter, 1)
            self.assertEqual(AppKit.NSTextAlignmentRight, 2)

        self.assertEqual(AppKit.NSTextAlignmentJustified, 3)
        self.assertEqual(AppKit.NSTextAlignmentNatural, 4)

        self.assertEqual(AppKit.NSWritingDirectionNatural, -1)
        self.assertEqual(AppKit.NSWritingDirectionLeftToRight, 0)
        self.assertEqual(AppKit.NSWritingDirectionRightToLeft, 1)

        self.assertEqual(AppKit.NSIllegalTextMovement, 0)
        self.assertEqual(AppKit.NSReturnTextMovement, 0x10)
        self.assertEqual(AppKit.NSTabTextMovement, 0x11)
        self.assertEqual(AppKit.NSBacktabTextMovement, 0x12)
        self.assertEqual(AppKit.NSLeftTextMovement, 0x13)
        self.assertEqual(AppKit.NSRightTextMovement, 0x14)
        self.assertEqual(AppKit.NSUpTextMovement, 0x15)
        self.assertEqual(AppKit.NSDownTextMovement, 0x16)
        self.assertEqual(AppKit.NSCancelTextMovement, 0x17)
        self.assertEqual(AppKit.NSOtherTextMovement, 0)

        self.assertIsInstance(AppKit.NSTextDidBeginEditingNotification, str)
        self.assertIsInstance(AppKit.NSTextDidEndEditingNotification, str)
        self.assertIsInstance(AppKit.NSTextDidChangeNotification, str)

        self.assertEqual(AppKit.NSTextMovementReturn, 0x10)
        self.assertEqual(AppKit.NSTextMovementTab, 0x11)
        self.assertEqual(AppKit.NSTextMovementBacktab, 0x12)
        self.assertEqual(AppKit.NSTextMovementLeft, 0x13)
        self.assertEqual(AppKit.NSTextMovementRight, 0x14)
        self.assertEqual(AppKit.NSTextMovementUp, 0x15)
        self.assertEqual(AppKit.NSTextMovementDown, 0x16)
        self.assertEqual(AppKit.NSTextMovementCancel, 0x17)
        self.assertEqual(AppKit.NSTextMovementOther, 0)

    @min_os_level("10.6")
    def testConstants10_6(self):
        self.assertEqual(AppKit.NSTextWritingDirectionEmbedding, 0 << 1)
        self.assertEqual(AppKit.NSTextWritingDirectionOverride, 1 << 1)

    @min_os_level("10.13")
    def testConstants10_13(self):
        self.assertIsInstance(AppKit.NSTextMovementUserInfoKey, str)

    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSText.writeRTFDToFile_atomically_)
        self.assertArgIsBOOL(AppKit.NSText.writeRTFDToFile_atomically_, 1)
        self.assertResultIsBOOL(AppKit.NSText.readRTFDFromFile_)
        self.assertResultIsBOOL(AppKit.NSText.isEditable)
        self.assertArgIsBOOL(AppKit.NSText.setEditable_, 0)
        self.assertResultIsBOOL(AppKit.NSText.isSelectable)
        self.assertArgIsBOOL(AppKit.NSText.setSelectable_, 0)
        self.assertResultIsBOOL(AppKit.NSText.isRichText)
        self.assertArgIsBOOL(AppKit.NSText.setRichText_, 0)
        self.assertResultIsBOOL(AppKit.NSText.importsGraphics)
        self.assertArgIsBOOL(AppKit.NSText.setImportsGraphics_, 0)
        self.assertResultIsBOOL(AppKit.NSText.isFieldEditor)
        self.assertArgIsBOOL(AppKit.NSText.setFieldEditor_, 0)
        self.assertResultIsBOOL(AppKit.NSText.usesFontPanel)
        self.assertArgIsBOOL(AppKit.NSText.setUsesFontPanel_, 0)
        self.assertResultIsBOOL(AppKit.NSText.drawsBackground)
        self.assertArgIsBOOL(AppKit.NSText.setDrawsBackground_, 0)
        self.assertResultIsBOOL(AppKit.NSText.isRulerVisible)
        self.assertResultIsBOOL(AppKit.NSText.isHorizontallyResizable)
        self.assertArgIsBOOL(AppKit.NSText.setHorizontallyResizable_, 0)
        self.assertResultIsBOOL(AppKit.NSText.isVerticallyResizable)
        self.assertArgIsBOOL(AppKit.NSText.setVerticallyResizable_, 0)

    @min_sdk_level("10.6")
    def testProtocolObjects(self):
        objc.protocolNamed("NSTextDelegate")

    def testProtocols(self):
        self.assertResultIsBOOL(TestNSTextHelper.textShouldBeginEditing_)
        self.assertResultIsBOOL(TestNSTextHelper.textShouldEndEditing_)



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSOpenGLLayer(TestCase):
    @min_os_level("10.6")
    def testMethods10_6(self):
        self.assertResultIsBOOL(
            AppKit.NSOpenGLLayer.canDrawInOpenGLContext_pixelFormat_forLayerTime_displayTime_
        )
        self.assertArgIsIn(
            AppKit.NSOpenGLLayer.canDrawInOpenGLContext_pixelFormat_forLayerTime_displayTime_,
            3,
        )

        self.assertArgIsIn(
            AppKit.NSOpenGLLayer.drawInOpenGLContext_pixelFormat_forLayerTime_displayTime_,
            3,
        )



import sys

import objc
from AppKit import NSApplicationMain, NSObject


class Converter(NSObject):
    def convertAmount_atRate_(self, amt, rate):
        return amt * rate


class ConverterController(NSObject):

    # First define the IB Outlets, the 'ivar' calls below define new
    # instance variables in the objective-C class (e.g. visible
    # for introspection in objective-C)
    converter = objc.IBOutlet()
    dollarField = objc.IBOutlet()
    rateField = objc.IBOutlet()
    totalField = objc.IBOutlet()

    def awakeFromNib(self):
        # Provide some defaults for the user...
        self.dollarField.setFloatValue_(2.0)
        self.rateField.setFloatValue_(3.0)

    @objc.IBAction
    def convert_(self, sender):
        rate = self.rateField.floatValue()
        amt = self.dollarField.floatValue()

        total = self.converter.convertAmount_atRate_(rate, amt)
        self.totalField.setFloatValue_(total)
        self.rateField.selectText_(self)

        # x = NSRunAlertPanel("Calculation Result",
        #    "The result is %s"%(total), "OK", None, None)


sys.exit(NSApplicationMain(sys.argv))



import AppKit
from PyObjCTools.TestSupport import TestCase


class TestNSBox(TestCase):
    def testConstants(self):
        self.assertEqual(AppKit.NSNoTitle, 0)
        self.assertEqual(AppKit.NSAboveTop, 1)
        self.assertEqual(AppKit.NSAtTop, 2)
        self.assertEqual(AppKit.NSBelowTop, 3)
        self.assertEqual(AppKit.NSAboveBottom, 4)
        self.assertEqual(AppKit.NSAtBottom, 5)
        self.assertEqual(AppKit.NSBelowBottom, 6)

        self.assertEqual(AppKit.NSBoxPrimary, 0)
        self.assertEqual(AppKit.NSBoxSecondary, 1)
        self.assertEqual(AppKit.NSBoxSeparator, 2)
        self.assertEqual(AppKit.NSBoxOldStyle, 3)
        self.assertEqual(AppKit.NSBoxCustom, 4)

    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSBox.isTransparent)
        self.assertArgIsBOOL(AppKit.NSBox.setTransparent_, 0)



import AppKit
from PyObjCTools.TestSupport import TestCase
import objc


class TestNSGlyphGeneratorHelper(AppKit.NSObject):
    def insertGlyphs_length_forStartingGlyphAtIndex_characterIndex_(
        self, glyphs, length, glyphIndex, charIndex
    ):
        self.glyphs = (glyphs, length, glyphIndex, charIndex)

    def setIntAttribute_value_forGlyphAtIndex_(self, a, v, g):
        pass


class TestNSGlyphGenerator(TestCase):
    def testConstants(self):
        self.assertEqual(AppKit.NSShowControlGlyphs, (1 << 0))
        self.assertEqual(AppKit.NSShowInvisibleGlyphs, (1 << 1))
        self.assertEqual(AppKit.NSWantsBidiLevels, (1 << 2))

    def testProtocols(self):
        objc.protocolNamed("NSGlyphStorage")
        self.assertArgHasType(
            TestNSGlyphGeneratorHelper.setIntAttribute_value_forGlyphAtIndex_,
            0,
            objc._C_NSInteger,
        )
        self.assertArgHasType(
            TestNSGlyphGeneratorHelper.setIntAttribute_value_forGlyphAtIndex_,
            1,
            objc._C_NSInteger,
        )
        self.assertArgHasType(
            TestNSGlyphGeneratorHelper.setIntAttribute_value_forGlyphAtIndex_,
            2,
            objc._C_NSUInteger,
        )

        o = TestNSGlyphGeneratorHelper.alloc().init()
        o.insertGlyphs_length_forStartingGlyphAtIndex_characterIndex_(
            [0, 1, 2, 3, 4], 5, 3, 8
        )
        self.assertEqual(o.glyphs, ([0, 1, 2, 3, 4], 5, 3, 8))
        self.assertArgHasType(
            TestNSGlyphGeneratorHelper.insertGlyphs_length_forStartingGlyphAtIndex_characterIndex_,  # noqa: B950
            0,
            b"n^I",
        )
        self.assertArgSizeInArg(
            TestNSGlyphGeneratorHelper.insertGlyphs_length_forStartingGlyphAtIndex_characterIndex_,  # noqa: B950
            0,
            1,
        )
        self.assertArgHasType(
            TestNSGlyphGeneratorHelper.insertGlyphs_length_forStartingGlyphAtIndex_characterIndex_,  # noqa: B950
            1,
            objc._C_NSUInteger,
        )
        self.assertArgHasType(
            TestNSGlyphGeneratorHelper.insertGlyphs_length_forStartingGlyphAtIndex_characterIndex_,  # noqa: B950
            2,
            objc._C_NSUInteger,
        )
        self.assertArgHasType(
            TestNSGlyphGeneratorHelper.insertGlyphs_length_forStartingGlyphAtIndex_characterIndex_,  # noqa: B950
            3,
            objc._C_NSUInteger,
        )

    def testMethods(self):
        self.assertArgIsOut(
            AppKit.NSGlyphGenerator.generateGlyphsForGlyphStorage_desiredNumberOfCharacters_glyphIndex_characterIndex_,  # noqa: B950
            2,
        )
        self.assertArgIsOut(
            AppKit.NSGlyphGenerator.generateGlyphsForGlyphStorage_desiredNumberOfCharacters_glyphIndex_characterIndex_,  # noqa: B950
            3,
        )



import objc
import Foundation
from PyObjCTools.TestSupport import TestCase


class TestNSLockProtocols(TestCase):
    def testLockIsLock(self):
        # Test for bug #1735937
        lock = Foundation.NSLock.alloc().init()
        self.assertTrue(lock.conformsToProtocol_(objc.protocolNamed("NSLocking")))

    def testMethods(self):
        self.assertResultIsBOOL(Foundation.NSLock.tryLock)
        self.assertResultIsBOOL(Foundation.NSLock.lockBeforeDate_)

        self.assertResultIsBOOL(Foundation.NSConditionLock.tryLock)
        self.assertResultIsBOOL(Foundation.NSConditionLock.tryLockWhenCondition_)
        self.assertResultIsBOOL(Foundation.NSConditionLock.lockBeforeDate_)
        self.assertResultIsBOOL(
            Foundation.NSConditionLock.lockWhenCondition_beforeDate_
        )

        self.assertResultIsBOOL(Foundation.NSRecursiveLock.tryLock)
        self.assertResultIsBOOL(Foundation.NSRecursiveLock.lockBeforeDate_)

        self.assertResultIsBOOL(Foundation.NSCondition.waitUntilDate_)



import AppKit
import objc
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSViewController(TestCase):
    @min_os_level("10.5")
    def testMethods(self):
        self.assertArgIsSEL(
            AppKit.NSViewController.commitEditingWithDelegate_didCommitSelector_contextInfo_,
            1,
            b"v@:@" + objc._C_NSBOOL + b"^v",
        )
        self.assertArgHasType(
            AppKit.NSViewController.commitEditingWithDelegate_didCommitSelector_contextInfo_,
            2,
            b"^v",
        )

        self.assertResultIsBOOL(AppKit.NSViewController.commitEditing)

    @min_os_level("10.10")
    def testMethods10_10(self):
        self.assertResultIsBOOL(AppKit.NSViewController.isViewLoaded)

        self.assertArgIsBlock(
            AppKit.NSViewController.transitionFromViewController_toViewController_options_completionHandler_,  # noqa: B950
            3,
            b"v",
        )

    @min_os_level("10.10")
    def testProtocols10_10(self):
        objc.protocolNamed("NSViewControllerPresentationAnimator")

    @min_os_level("10.10")
    def testConstants10_10(self):
        self.assertEqual(AppKit.NSViewControllerTransitionNone, 0x0)
        self.assertEqual(AppKit.NSViewControllerTransitionCrossfade, 0x1)
        self.assertEqual(AppKit.NSViewControllerTransitionSlideUp, 0x10)
        self.assertEqual(AppKit.NSViewControllerTransitionSlideDown, 0x20)
        self.assertEqual(AppKit.NSViewControllerTransitionSlideLeft, 0x40)
        self.assertEqual(AppKit.NSViewControllerTransitionSlideRight, 0x80)
        self.assertEqual(AppKit.NSViewControllerTransitionSlideForward, 0x140)
        self.assertEqual(AppKit.NSViewControllerTransitionSlideBackward, 0x180)
        self.assertEqual(AppKit.NSViewControllerTransitionAllowUserInteraction, 0x1000)



import Foundation
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSDateComponentsFormatter(TestCase):
    @min_os_level("10.10")
    def testConstants10_10(self):
        self.assertEqual(Foundation.NSDateComponentsFormatterUnitsStylePositional, 0)
        self.assertEqual(Foundation.NSDateComponentsFormatterUnitsStyleAbbreviated, 1)
        self.assertEqual(Foundation.NSDateComponentsFormatterUnitsStyleShort, 2)
        self.assertEqual(Foundation.NSDateComponentsFormatterUnitsStyleFull, 3)
        self.assertEqual(Foundation.NSDateComponentsFormatterUnitsStyleSpellOut, 4)
        self.assertEqual(Foundation.NSDateComponentsFormatterUnitsStyleBrief, 5)

        self.assertEqual(
            Foundation.NSDateComponentsFormatterZeroFormattingBehaviorNone, 0
        )
        self.assertEqual(
            Foundation.NSDateComponentsFormatterZeroFormattingBehaviorDefault, 1 << 0
        )
        self.assertEqual(
            Foundation.NSDateComponentsFormatterZeroFormattingBehaviorDropLeading,
            1 << 1,
        )
        self.assertEqual(
            Foundation.NSDateComponentsFormatterZeroFormattingBehaviorDropMiddle, 1 << 2
        )
        self.assertEqual(
            Foundation.NSDateComponentsFormatterZeroFormattingBehaviorDropTrailing,
            1 << 3,
        )
        self.assertEqual(
            Foundation.NSDateComponentsFormatterZeroFormattingBehaviorDropAll,
            Foundation.NSDateComponentsFormatterZeroFormattingBehaviorDropLeading
            | Foundation.NSDateComponentsFormatterZeroFormattingBehaviorDropMiddle
            | Foundation.NSDateComponentsFormatterZeroFormattingBehaviorDropTrailing,
        )

        self.assertEqual(
            Foundation.NSDateComponentsFormatterZeroFormattingBehaviorPad, 1 << 16
        )

    @min_os_level("10.10")
    def testMethods10_10(self):
        self.assertResultIsBOOL(
            Foundation.NSDateComponentsFormatter.allowsFractionalUnits
        )
        self.assertArgIsBOOL(
            Foundation.NSDateComponentsFormatter.setAllowsFractionalUnits_, 0
        )
        self.assertResultIsBOOL(
            Foundation.NSDateComponentsFormatter.collapsesLargestUnit
        )
        self.assertArgIsBOOL(
            Foundation.NSDateComponentsFormatter.setCollapsesLargestUnit_, 0
        )
        self.assertResultIsBOOL(
            Foundation.NSDateComponentsFormatter.includesApproximationPhrase
        )
        self.assertArgIsBOOL(
            Foundation.NSDateComponentsFormatter.setIncludesApproximationPhrase_, 0
        )
        self.assertResultIsBOOL(
            Foundation.NSDateComponentsFormatter.includesTimeRemainingPhrase
        )
        self.assertArgIsBOOL(
            Foundation.NSDateComponentsFormatter.setIncludesTimeRemainingPhrase_, 0
        )

        self.assertResultIsBOOL(
            Foundation.NSDateComponentsFormatter.getObjectValue_forString_errorDescription_
        )
        self.assertArgIsOut(
            Foundation.NSDateComponentsFormatter.getObjectValue_forString_errorDescription_,
            0,
        )
        self.assertArgIsOut(
            Foundation.NSDateComponentsFormatter.getObjectValue_forString_errorDescription_,
            2,
        )



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level

try:
    import Quartz
except ImportError:
    Quartz = None


class TestNSGraphicsContext(TestCase):
    def testConstants(self):
        self.assertIsInstance(AppKit.NSGraphicsContextDestinationAttributeName, str)
        self.assertIsInstance(
            AppKit.NSGraphicsContextRepresentationFormatAttributeName, str
        )
        self.assertIsInstance(AppKit.NSGraphicsContextPSFormat, str)
        self.assertIsInstance(AppKit.NSGraphicsContextPDFFormat, str)

        self.assertEqual(AppKit.NSImageInterpolationDefault, 0)
        self.assertEqual(AppKit.NSImageInterpolationNone, 1)
        self.assertEqual(AppKit.NSImageInterpolationLow, 2)
        self.assertEqual(AppKit.NSImageInterpolationHigh, 3)
        self.assertEqual(AppKit.NSImageInterpolationMedium, 4)

        self.assertEqual(AppKit.NSColorRenderingIntentDefault, 0)
        self.assertEqual(AppKit.NSColorRenderingIntentAbsoluteColorimetric, 1)
        self.assertEqual(AppKit.NSColorRenderingIntentRelativeColorimetric, 2)
        self.assertEqual(AppKit.NSColorRenderingIntentPerceptual, 3)
        self.assertEqual(AppKit.NSColorRenderingIntentSaturation, 4)

    def testMethods(self):
        self.assertArgIsBOOL(
            AppKit.NSGraphicsContext.graphicsContextWithGraphicsPort_flipped_, 1
        )
        self.assertResultIsBOOL(AppKit.NSGraphicsContext.currentContextDrawingToScreen)
        self.assertResultIsBOOL(AppKit.NSGraphicsContext.isDrawingToScreen)
        self.assertResultIsBOOL(AppKit.NSGraphicsContext.isFlipped)
        self.assertResultIsBOOL(AppKit.NSGraphicsContext.shouldAntialias)
        self.assertArgIsBOOL(AppKit.NSGraphicsContext.setShouldAntialias_, 0)

        img = AppKit.NSBitmapImageRep.alloc().initWithBitmapDataPlanes_pixelsWide_pixelsHigh_bitsPerSample_samplesPerPixel_hasAlpha_isPlanar_colorSpaceName_bitmapFormat_bytesPerRow_bitsPerPixel_(  # noqa: B950
            None, 255, 255, 8, 4, True, False, AppKit.NSCalibratedRGBColorSpace, 0, 0, 0
        )
        context = AppKit.NSGraphicsContext.graphicsContextWithBitmapImageRep_(img)
        self.assertIsInstance(context, AppKit.NSGraphicsContext)
        if Quartz is not None:
            port = context.graphicsPort()
            self.assertIsInstance(port, Quartz.CGContextRef)

        self.assertArgHasType(
            AppKit.NSGraphicsContext.graphicsContextWithGraphicsPort_flipped_,
            0,
            b"^{CGContext=}",
        )
        self.assertResultHasType(
            AppKit.NSGraphicsContext.graphicsPort, b"^{CGContext=}"
        )

    @min_os_level("10.10")
    def testMethods10_10(self):
        self.assertArgIsBOOL(
            AppKit.NSGraphicsContext.graphicsContextWithCGContext_flipped_, 1
        )



import Cocoa
from PyObjCTools.TestSupport import TestCase

# Would like some tests for NSRunAlertPanel and friends as well, but those
# require user interaction :-(


class TestAlertFormat(TestCase):
    def testSimple(self):
        alert = Cocoa.NSAlert.alertWithMessageText_defaultButton_alternateButton_otherButton_informativeTextWithFormat_(  # noqa: B950
            "message text", "ok", "cancel", "help", "foobar is the sucks"
        )
        self.assertEqual(alert.messageText(), "message text")
        self.assertEqual(alert.informativeText(), "foobar is the sucks")

    def testWithFormat(self):
        alert = Cocoa.NSAlert.alertWithMessageText_defaultButton_alternateButton_otherButton_informativeTextWithFormat_(  # noqa: B950
            "message text", "ok", "cancel", "help", "%d * %d = %d", 9, 7, 9 * 7
        )
        self.assertEqual(alert.messageText(), "message text")
        self.assertEqual(alert.informativeText(), "9 * 7 = 63")



import Foundation
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSScannerUsage(TestCase):
    def testUsage(self):
        obj = Foundation.NSScanner.scannerWithString_("1.2 2.5")

        didConvert, value = obj.scanDouble_(None)
        self.assertTrue(didConvert)
        self.assertAlmostEqual(value, 1.2)

        didConvert, value = obj.scanFloat_(None)
        self.assertTrue(didConvert)
        self.assertAlmostEqual(value, 2.5)

        obj = Foundation.NSScanner.scannerWithString_("abcd1234 efgh")

        didConvert, value = obj.scanCharactersFromSet_intoString_(
            Foundation.NSCharacterSet.lowercaseLetterCharacterSet(), None
        )
        self.assertTrue(didConvert)
        self.assertEqual(value, "abcd")

        didConvert, value = obj.scanInt_(None)
        self.assertTrue(didConvert)
        self.assertEqual(value, 1234)

        obj = Foundation.NSScanner.scannerWithString_("1234 efgh")

        didConvert, value = obj.scanLongLong_(None)
        self.assertTrue(didConvert)
        self.assertEqual(value, 1234)

        didConvert, value = obj.scanString_intoString_("efgh", None)
        self.assertTrue(didConvert)
        self.assertEqual(value, "efgh")

        obj = Foundation.NSScanner.scannerWithString_("1234 efgh")
        didConvert, value = obj.scanUpToCharactersFromSet_intoString_(
            Foundation.NSCharacterSet.lowercaseLetterCharacterSet(), None
        )
        self.assertTrue(didConvert)
        self.assertEqual(value, "1234 ")

    def testMethods(self):
        self.assertArgIsBOOL(Foundation.NSScanner.setCaseSensitive_, 0)
        self.assertResultIsBOOL(Foundation.NSScanner.caseSensitive)
        self.assertResultIsBOOL(Foundation.NSScanner.scanInt_)
        self.assertArgIsOut(Foundation.NSScanner.scanInt_, 0)
        self.assertResultIsBOOL(Foundation.NSScanner.scanInteger_)
        self.assertArgIsOut(Foundation.NSScanner.scanInteger_, 0)
        self.assertResultIsBOOL(Foundation.NSScanner.scanHexLongLong_)
        self.assertArgIsOut(Foundation.NSScanner.scanHexLongLong_, 0)
        self.assertResultIsBOOL(Foundation.NSScanner.scanHexFloat_)
        self.assertArgIsOut(Foundation.NSScanner.scanHexFloat_, 0)
        self.assertResultIsBOOL(Foundation.NSScanner.scanHexDouble_)
        self.assertArgIsOut(Foundation.NSScanner.scanHexDouble_, 0)
        self.assertResultIsBOOL(Foundation.NSScanner.scanHexInt_)
        self.assertArgIsOut(Foundation.NSScanner.scanHexInt_, 0)
        self.assertResultIsBOOL(Foundation.NSScanner.scanLongLong_)
        self.assertArgIsOut(Foundation.NSScanner.scanLongLong_, 0)
        self.assertResultIsBOOL(Foundation.NSScanner.scanFloat_)
        self.assertArgIsOut(Foundation.NSScanner.scanFloat_, 0)
        self.assertResultIsBOOL(Foundation.NSScanner.scanDouble_)
        self.assertArgIsOut(Foundation.NSScanner.scanDouble_, 0)
        self.assertResultIsBOOL(Foundation.NSScanner.scanString_intoString_)
        self.assertArgIsOut(Foundation.NSScanner.scanString_intoString_, 1)
        self.assertResultIsBOOL(Foundation.NSScanner.scanCharactersFromSet_intoString_)
        self.assertArgIsOut(Foundation.NSScanner.scanCharactersFromSet_intoString_, 1)
        self.assertResultIsBOOL(Foundation.NSScanner.scanUpToString_intoString_)
        self.assertArgIsOut(Foundation.NSScanner.scanUpToString_intoString_, 1)
        self.assertResultIsBOOL(
            Foundation.NSScanner.scanUpToCharactersFromSet_intoString_
        )
        self.assertArgIsOut(
            Foundation.NSScanner.scanUpToCharactersFromSet_intoString_, 1
        )
        self.assertResultIsBOOL(Foundation.NSScanner.isAtEnd)

    @min_os_level("10.9")
    def testMethods10_9(self):
        self.assertResultIsBOOL(Foundation.NSScanner.scanUnsignedLongLong_)
        self.assertArgIsOut(Foundation.NSScanner.scanUnsignedLongLong_, 0)



import AppKit
from PyObjCTools.TestSupport import TestCase


class TestNSUserInterfaceLayout(TestCase):
    def testConstants(self):
        self.assertEqual(AppKit.NSUserInterfaceLayoutDirectionLeftToRight, 0)
        self.assertEqual(AppKit.NSUserInterfaceLayoutDirectionRightToLeft, 1)
        self.assertEqual(AppKit.NSUserInterfaceLayoutOrientationHorizontal, 0)
        self.assertEqual(AppKit.NSUserInterfaceLayoutOrientationVertical, 1)



import Foundation
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSExpression(TestCase):
    def testConstants(self):
        self.assertEqual(Foundation.NSConstantValueExpressionType, 0)
        self.assertEqual(Foundation.NSEvaluatedObjectExpressionType, 1)
        self.assertEqual(Foundation.NSVariableExpressionType, 2)
        self.assertEqual(Foundation.NSKeyPathExpressionType, 3)
        self.assertEqual(Foundation.NSFunctionExpressionType, 4)
        self.assertEqual(Foundation.NSUnionSetExpressionType, 5)
        self.assertEqual(Foundation.NSIntersectSetExpressionType, 6)
        self.assertEqual(Foundation.NSMinusSetExpressionType, 7)
        self.assertEqual(Foundation.NSSubqueryExpressionType, 13)
        self.assertEqual(Foundation.NSAggregateExpressionType, 14)

    @min_os_level("10.6")
    def testConstants10_6(self):
        self.assertEqual(Foundation.NSBlockExpressionType, 19)

    @min_os_level("10.9")
    def testConstants10_9(self):
        self.assertEqual(Foundation.NSAnyKeyExpressionType, 15)

    @min_os_level("10.11")
    def testConstants10_11(self):
        self.assertEqual(Foundation.NSConditionalExpressionType, 20)

    @min_os_level("10.6")
    def testMethods10_6(self):
        self.assertArgIsBlock(
            Foundation.NSExpression.expressionForBlock_arguments_, 0, b"@@@@"
        )
        self.assertResultIsBlock(Foundation.NSExpression.expressionBlock, b"@@@@")

    @min_os_level("10.6")
    def testMethod10_6_unsupported(self):
        self.assertArgIsPrintf(Foundation.NSExpression.expressionWithFormat_, 0)



# NOTE: This file only contains basic tests of the keyvalue observing header definitions,
# test_keyvalue contains tests for the actually KVC/KVO mechanisms.
import Foundation
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSKeyValueObserving(TestCase):
    def testConstants(self):
        self.assertEqual(Foundation.NSKeyValueObservingOptionNew, 1)
        self.assertEqual(Foundation.NSKeyValueObservingOptionOld, 2)
        self.assertEqual(Foundation.NSKeyValueObservingOptionInitial, 4)
        self.assertEqual(Foundation.NSKeyValueObservingOptionPrior, 8)

        self.assertEqual(Foundation.NSKeyValueChangeSetting, 1)
        self.assertEqual(Foundation.NSKeyValueChangeInsertion, 2)
        self.assertEqual(Foundation.NSKeyValueChangeRemoval, 3)
        self.assertEqual(Foundation.NSKeyValueChangeReplacement, 4)

        self.assertEqual(Foundation.NSKeyValueUnionSetMutation, 1)
        self.assertEqual(Foundation.NSKeyValueMinusSetMutation, 2)
        self.assertEqual(Foundation.NSKeyValueIntersectSetMutation, 3)
        self.assertEqual(Foundation.NSKeyValueSetSetMutation, 4)

        self.assertIsInstance(Foundation.NSKeyValueChangeKindKey, str)
        self.assertIsInstance(Foundation.NSKeyValueChangeNewKey, str)
        self.assertIsInstance(Foundation.NSKeyValueChangeOldKey, str)
        self.assertIsInstance(Foundation.NSKeyValueChangeIndexesKey, str)
        self.assertIsInstance(Foundation.NSKeyValueChangeNotificationIsPriorKey, str)

    def testContext(self):
        o = Foundation.NSObject.alloc().init()
        m = o.observeValueForKeyPath_ofObject_change_context_.__metadata__()
        self.assertEqual(m["arguments"][5]["type"], b"^v")

        m = o.addObserver_forKeyPath_options_context_.__metadata__()
        self.assertEqual(m["arguments"][5]["type"], b"^v")

        m = o.setObservationInfo_.__metadata__()
        self.assertEqual(m["arguments"][2]["type"], b"^v")

        m = o.observationInfo.__metadata__()
        self.assertEqual(m["retval"]["type"], b"^v")

        o = Foundation.NSArray.alloc().init()
        m = o.addObserver_toObjectsAtIndexes_forKeyPath_options_context_.__metadata__()
        self.assertEqual(m["arguments"][6]["type"], b"^v")

        m = o.addObserver_forKeyPath_options_context_.__metadata__()
        self.assertEqual(m["arguments"][5]["type"], b"^v")

        o = Foundation.NSSet.alloc().init()
        m = o.addObserver_forKeyPath_options_context_.__metadata__()
        self.assertEqual(m["arguments"][5]["type"], b"^v")

    @min_os_level("10.7")
    def testContext10_7(self):
        o = Foundation.NSObject.alloc().init()
        m = o.removeObserver_forKeyPath_context_.__metadata__()
        self.assertEqual(m["arguments"][4]["type"], b"^v")

        o = Foundation.NSMutableArray.alloc().init()
        m = o.removeObserver_fromObjectsAtIndexes_forKeyPath_context_.__metadata__()
        self.assertEqual(m["arguments"][5]["type"], b"^v")

        o = Foundation.NSOrderedSet.alloc().init()
        m = o.removeObserver_forKeyPath_context_.__metadata__()
        self.assertEqual(m["arguments"][4]["type"], b"^v")

        o = Foundation.NSSet.alloc().init()
        m = o.removeObserver_forKeyPath_context_.__metadata__()
        self.assertEqual(m["arguments"][4]["type"], b"^v")

    def testMethods(self):
        self.assertResultIsBOOL(
            Foundation.NSObject.automaticallyNotifiesObserversForKey_
        )



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level, min_sdk_level
import objc


class TestNSControlHelper(AppKit.NSObject):
    def control_textShouldBeginEditing_(self, c, f):
        return 1

    def control_textShouldEndEditing_(self, c, f):
        return 1

    def control_didFailToFormatString_errorDescription_(self, c, s, e):
        return 1

    def control_isValidObject_(self, c, s):
        return 1

    def control_textView_doCommandBySelector_(self, c, t, com):
        return 1

    def control_textView_completions_forPartialWordRange_indexOfSelectedItem_(
        self, c, t, c1, r, i
    ):
        return 1


class TestNSControl(TestCase):
    @min_os_level("10.10")
    def testMethods10_10(self):
        self.assertResultIsBOOL(AppKit.NSControl.usesSingleLineMode)
        self.assertArgIsBOOL(AppKit.NSControl.setUsesSingleLineMode_, 0)

        self.assertResultIsBOOL(AppKit.NSControl.isHighlighted)
        self.assertArgIsBOOL(AppKit.NSControl.setHighlighted_, 0)

    @min_os_level("10.8")
    def testMethods10_8(self):
        self.assertResultIsBOOL(AppKit.NSControl.allowsExpansionToolTips)
        self.assertArgIsBOOL(AppKit.NSControl.setAllowsExpansionToolTips_, 0)

    def testMethods(self):
        self.assertArgIsSEL(AppKit.NSControl.setAction_, 0, b"v@:@")
        self.assertResultIsBOOL(AppKit.NSControl.ignoresMultiClick)
        self.assertArgIsBOOL(AppKit.NSControl.setIgnoresMultiClick_, 0)
        self.assertResultIsBOOL(AppKit.NSControl.isContinuous)
        self.assertArgIsBOOL(AppKit.NSControl.setContinuous_, 0)
        self.assertResultIsBOOL(AppKit.NSControl.isEnabled)
        self.assertArgIsBOOL(AppKit.NSControl.setEnabled_, 0)
        self.assertArgIsBOOL(AppKit.NSControl.setFloatingPointFormat_left_right_, 0)
        self.assertResultIsBOOL(AppKit.NSControl.sendAction_to_)
        self.assertArgIsSEL(AppKit.NSControl.sendAction_to_, 0, b"v@:@")
        self.assertResultIsBOOL(AppKit.NSControl.abortEditing)
        self.assertResultIsBOOL(AppKit.NSControl.refusesFirstResponder)
        self.assertArgIsBOOL(AppKit.NSControl.setRefusesFirstResponder_, 0)

    @min_sdk_level("10.6")
    def testProtocols(self):
        objc.protocolNamed("NSControlTextEditingDelegate")

    def testDelegate(self):
        self.assertResultIsBOOL(TestNSControlHelper.control_textShouldBeginEditing_)
        self.assertResultIsBOOL(TestNSControlHelper.control_textShouldEndEditing_)
        self.assertResultIsBOOL(
            TestNSControlHelper.control_didFailToFormatString_errorDescription_
        )
        self.assertResultIsBOOL(TestNSControlHelper.control_isValidObject_)
        self.assertResultIsBOOL(
            TestNSControlHelper.control_textView_doCommandBySelector_
        )

        self.assertArgHasType(
            TestNSControlHelper.control_textView_completions_forPartialWordRange_indexOfSelectedItem_,  # noqa: B950
            3,
            AppKit.NSRange.__typestr__,
        )
        self.assertArgHasType(
            TestNSControlHelper.control_textView_completions_forPartialWordRange_indexOfSelectedItem_,  # noqa: B950
            4,
            b"N^" + objc._C_NSInteger,
        )

    def testConstants(self):
        self.assertIsInstance(AppKit.NSControlTextDidBeginEditingNotification, str)
        self.assertIsInstance(AppKit.NSControlTextDidEndEditingNotification, str)
        self.assertIsInstance(AppKit.NSControlTextDidChangeNotification, str)



import AppKit
from PyObjCTools.TestSupport import TestCase


class TestNSecureTextFieldCell(TestCase):
    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSSecureTextFieldCell.echosBullets)
        self.assertArgIsBOOL(AppKit.NSSecureTextFieldCell.setEchosBullets_, 0)



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSAnimationContext(TestCase):
    @min_os_level("10.7")
    def testMethods10_7(self):
        self.assertResultIsBlock(AppKit.NSAnimationContext.completionHandler, b"v")
        self.assertArgIsBlock(AppKit.NSAnimationContext.setCompletionHandler_, 0, b"v")

        self.assertArgIsBlock(
            AppKit.NSAnimationContext.runAnimationGroup_completionHandler_, 0, b"v@"
        )
        self.assertArgIsBlock(
            AppKit.NSAnimationContext.runAnimationGroup_completionHandler_, 1, b"v"
        )

    @min_os_level("10.8")
    def testMethods10_8(self):
        self.assertResultIsBOOL(AppKit.NSAnimationContext.allowsImplicitAnimation)
        self.assertArgIsBOOL(AppKit.NSAnimationContext.setAllowsImplicitAnimation_, 0)



import Foundation  # noqa: F401
from PyObjCTools.TestSupport import TestCase
import objc


class TestNSURLAuthenticationChallenge(TestCase):
    def testProtocols(self):
        objc.protocolNamed("NSURLAuthenticationChallengeSender")



import Foundation
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSFormatter(TestCase):
    @min_os_level("10.10")
    def testConstants10_10(self):
        self.assertEqual(Foundation.NSFormattingContextUnknown, 0)
        self.assertEqual(Foundation.NSFormattingContextDynamic, 1)
        self.assertEqual(Foundation.NSFormattingContextStandalone, 2)
        self.assertEqual(Foundation.NSFormattingContextListItem, 3)
        self.assertEqual(Foundation.NSFormattingContextBeginningOfSentence, 4)
        self.assertEqual(Foundation.NSFormattingContextMiddleOfSentence, 5)

        self.assertEqual(Foundation.NSFormattingUnitStyleShort, 1)
        self.assertEqual(Foundation.NSFormattingUnitStyleMedium, 2)
        self.assertEqual(Foundation.NSFormattingUnitStyleLong, 3)

    def testMethods(self):
        self.assertResultIsBOOL(
            Foundation.NSFormatter.getObjectValue_forString_errorDescription_
        )
        self.assertArgIsOut(
            Foundation.NSFormatter.getObjectValue_forString_errorDescription_, 0
        )
        self.assertArgIsOut(
            Foundation.NSFormatter.getObjectValue_forString_errorDescription_, 2
        )

        self.assertResultIsBOOL(
            Foundation.NSFormatter.isPartialStringValid_newEditingString_errorDescription_
        )
        self.assertArgIsInOut(
            Foundation.NSFormatter.isPartialStringValid_newEditingString_errorDescription_,
            1,
        )
        self.assertArgIsOut(
            Foundation.NSFormatter.isPartialStringValid_newEditingString_errorDescription_,
            2,
        )

        self.assertResultIsBOOL(
            Foundation.NSFormatter.isPartialStringValid_proposedSelectedRange_originalString_originalSelectedRange_errorDescription_  # noqa: B950
        )
        self.assertArgIsInOut(
            Foundation.NSFormatter.isPartialStringValid_proposedSelectedRange_originalString_originalSelectedRange_errorDescription_,  # noqa: B950
            0,
        )
        self.assertArgIsInOut(
            Foundation.NSFormatter.isPartialStringValid_proposedSelectedRange_originalString_originalSelectedRange_errorDescription_,  # noqa: B950
            1,
        )
        self.assertArgIsOut(
            Foundation.NSFormatter.isPartialStringValid_proposedSelectedRange_originalString_originalSelectedRange_errorDescription_,  # noqa: B950
            4,
        )



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level, min_sdk_level
import objc


class TestNSTextViewHelper(AppKit.NSObject):
    def textView_URLForContentsOfTextAttachment_atIndex_(self, a, b, c):
        pass

    def textView_clickedOnLink_atIndex_(self, tv, a, b):
        return 1

    def textView_clickedOnCell_inRect_atIndex_(self, tv, a, b, c):
        return 1

    def textView_doubleClickedOnCell_inRect_atIndex_(self, tv, a, b, c):
        return 1

    def textView_draggedCell_inRect_event_atIndex_(elf, tv, a, b, c, d):
        return 1

    def textView_writablePasteboardTypesForCell_atIndex_(self, tv, a, b):
        return 1

    def textView_writeCell_atIndex_toPasteboard_type_(self, tv, a, b, c, d):
        return 1

    def textView_willChangeSelectionFromCharacterRange_toCharacterRange_(
        self, tv, a, b
    ):
        return 1

    def textView_shouldChangeTextInRanges_replacementStrings_(self, tv, a, b):
        return 1

    def textView_willDisplayToolTip_forCharacterAtIndex_(self, tv, a, b):
        return 1

    def textView_completions_forPartialWordRange_indexOfSelectedItem_(
        self, tv, a, b, c
    ):
        return 1

    def textView_shouldChangeTextInRange_replacementString_(self, tv, a, b):
        return 1

    def textView_doCommandBySelector_(self, tv, a):
        return 1

    def textView_clickedOnLink_(self, tv, a):
        return 1

    def textView_clickedOnCell_inRect_(self, tv, a, b):
        return 1

    def textView_doubleClickedOnCell_inRect_(self, tv, a, b):
        return 1

    def textView_draggedCell_inRect_event_(self, tv, a, b, c):
        return 1

    def textView_shouldSetSpellingState_range_(self, tv, a, b):
        return 1

    def textView_menu_forEvent_atIndex_(self, tv, a, b, c):
        return 1

    def textView_willCheckTextInRange_options_types_(self, tv, a, b, c):
        return 1

    def textView_didCheckTextInRange_types_options_results_orthography_wordCount_(
        self, tv, r, t, o, rs, ort, wc
    ):
        return 1

    def textView_candidatesForSelectedRange_(self, tv, r):
        return 1

    def textView_candidates_forSelectedRange_(self, tv, c, r):
        return 1

    def textView_shouldSelectCandidateAtIndex_(self, tv, i):
        return 1


class TestNSTextView(TestCase):
    def testConstants(self):
        self.assertEqual(AppKit.NSSelectByCharacter, 0)
        self.assertEqual(AppKit.NSSelectByWord, 1)
        self.assertEqual(AppKit.NSSelectByParagraph, 2)

        self.assertEqual(AppKit.NSSelectionAffinityUpstream, 0)
        self.assertEqual(AppKit.NSSelectionAffinityDownstream, 1)

        self.assertEqual(AppKit.NSFindPanelActionShowFindPanel, 1)
        self.assertEqual(AppKit.NSFindPanelActionNext, 2)
        self.assertEqual(AppKit.NSFindPanelActionPrevious, 3)
        self.assertEqual(AppKit.NSFindPanelActionReplaceAll, 4)
        self.assertEqual(AppKit.NSFindPanelActionReplace, 5)
        self.assertEqual(AppKit.NSFindPanelActionReplaceAndFind, 6)
        self.assertEqual(AppKit.NSFindPanelActionSetFindString, 7)
        self.assertEqual(AppKit.NSFindPanelActionReplaceAllInSelection, 8)
        self.assertEqual(AppKit.NSFindPanelActionSelectAll, 9)
        self.assertEqual(AppKit.NSFindPanelActionSelectAllInSelection, 10)

        self.assertIsInstance(AppKit.NSFindPanelSearchOptionsPboardType, str)

        self.assertIsInstance(AppKit.NSFindPanelCaseInsensitiveSearch, str)
        self.assertIsInstance(AppKit.NSFindPanelSubstringMatch, str)

        self.assertEqual(AppKit.NSFindPanelSubstringMatchTypeContains, 0)
        self.assertEqual(AppKit.NSFindPanelSubstringMatchTypeStartsWith, 1)
        self.assertEqual(AppKit.NSFindPanelSubstringMatchTypeFullWord, 2)
        self.assertEqual(AppKit.NSFindPanelSubstringMatchTypeEndsWith, 3)

        self.assertIsInstance(AppKit.NSAllRomanInputSourcesLocaleIdentifier, str)

        self.assertIsInstance(
            AppKit.NSTextViewWillChangeNotifyingTextViewNotification, str
        )
        self.assertIsInstance(AppKit.NSTextViewDidChangeSelectionNotification, str)
        self.assertIsInstance(
            AppKit.NSTextViewDidChangeTypingAttributesNotification, str
        )

    @min_os_level("10.12")
    def testConstants10_12(self):
        self.assertIsInstance(AppKit.NSTouchBarItemIdentifierCharacterPicker, str)
        self.assertIsInstance(AppKit.NSTouchBarItemIdentifierTextColorPicker, str)
        self.assertIsInstance(AppKit.NSTouchBarItemIdentifierTextStyle, str)
        self.assertIsInstance(AppKit.NSTouchBarItemIdentifierTextAlignment, str)
        self.assertIsInstance(AppKit.NSTouchBarItemIdentifierTextList, str)
        self.assertIsInstance(AppKit.NSTouchBarItemIdentifierTextFormat, str)

    @min_os_level("12.0")
    def testConstants12_0(self):
        self.assertIsInstance(
            AppKit.NSTextViewWillSwitchToNSLayoutManagerNotification, str
        )
        self.assertIsInstance(
            AppKit.NSTextViewDidSwitchToNSLayoutManagerNotification, str
        )

    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSTextView.shouldDrawInsertionPoint)
        # self.assertArgIsBOOL(AppKit.NSTextView.setShouldDrawInsertionPoint_, 0)
        self.assertResultIsBOOL(AppKit.NSTextView.rulerView_shouldMoveMarker_)
        self.assertResultIsBOOL(AppKit.NSTextView.rulerView_shouldAddMarker_)
        self.assertResultIsBOOL(AppKit.NSTextView.rulerView_shouldRemoveMarker_)
        self.assertArgIsBOOL(
            AppKit.NSTextView.setNeedsDisplayInRect_avoidAdditionalLayout_, 1
        )
        self.assertArgIsBOOL(
            AppKit.NSTextView.drawInsertionPointInRect_color_turnedOn_, 2
        )
        self.assertArgIsOut(
            AppKit.NSTextView.completionsForPartialWordRange_indexOfSelectedItem_, 1
        )
        self.assertArgIsBOOL(
            AppKit.NSTextView.insertCompletion_forPartialWordRange_movement_isFinal_, 3
        )
        self.assertResultIsBOOL(AppKit.NSTextView.writeSelectionToPasteboard_type_)
        self.assertResultIsBOOL(AppKit.NSTextView.writeSelectionToPasteboard_types_)
        self.assertResultIsBOOL(AppKit.NSTextView.readSelectionFromPasteboard_type_)
        self.assertResultIsBOOL(AppKit.NSTextView.readSelectionFromPasteboard_)
        self.assertResultIsBOOL(
            AppKit.NSTextView.dragSelectionWithEvent_offset_slideBack_
        )
        self.assertArgIsBOOL(
            AppKit.NSTextView.dragSelectionWithEvent_offset_slideBack_, 2
        )
        self.assertArgIsBOOL(
            AppKit.NSTextView.setSelectedRanges_affinity_stillSelecting_, 2
        )
        self.assertArgIsBOOL(
            AppKit.NSTextView.setSelectedRange_affinity_stillSelecting_, 2
        )
        self.assertArgIsBOOL(
            AppKit.NSTextView.updateInsertionPointStateAndRestartTimer_, 0
        )
        self.assertResultIsBOOL(AppKit.NSTextView.acceptsGlyphInfo)
        self.assertArgIsBOOL(AppKit.NSTextView.setAcceptsGlyphInfo_, 0)
        self.assertArgIsBOOL(AppKit.NSTextView.setRulerVisible_, 0)
        self.assertResultIsBOOL(AppKit.NSTextView.usesRuler)
        self.assertArgIsBOOL(AppKit.NSTextView.setUsesRuler_, 0)
        self.assertResultIsBOOL(AppKit.NSTextView.isContinuousSpellCheckingEnabled)
        self.assertArgIsBOOL(AppKit.NSTextView.setContinuousSpellCheckingEnabled_, 0)
        self.assertResultIsBOOL(AppKit.NSTextView.isGrammarCheckingEnabled)
        self.assertArgIsBOOL(AppKit.NSTextView.setGrammarCheckingEnabled_, 0)
        self.assertResultIsBOOL(
            AppKit.NSTextView.shouldChangeTextInRanges_replacementStrings_
        )
        self.assertResultIsBOOL(
            AppKit.NSTextView.shouldChangeTextInRange_replacementString_
        )
        self.assertResultIsBOOL(AppKit.NSTextView.usesFindPanel)
        self.assertArgIsBOOL(AppKit.NSTextView.setUsesFindPanel_, 0)
        self.assertResultIsBOOL(AppKit.NSTextView.allowsDocumentBackgroundColorChange)
        self.assertArgIsBOOL(
            AppKit.NSTextView.setAllowsDocumentBackgroundColorChange_, 0
        )
        self.assertResultIsBOOL(AppKit.NSTextView.allowsUndo)
        self.assertArgIsBOOL(AppKit.NSTextView.setAllowsUndo_, 0)
        self.assertResultIsBOOL(AppKit.NSTextView.allowsImageEditing)
        self.assertArgIsBOOL(AppKit.NSTextView.setAllowsImageEditing_, 0)
        self.assertResultIsBOOL(AppKit.NSTextView.isEditable)
        self.assertArgIsBOOL(AppKit.NSTextView.setEditable_, 0)
        self.assertResultIsBOOL(AppKit.NSTextView.isSelectable)
        self.assertArgIsBOOL(AppKit.NSTextView.setSelectable_, 0)
        self.assertResultIsBOOL(AppKit.NSTextView.isRichText)
        self.assertArgIsBOOL(AppKit.NSTextView.setRichText_, 0)
        self.assertResultIsBOOL(AppKit.NSTextView.importsGraphics)
        self.assertArgIsBOOL(AppKit.NSTextView.setImportsGraphics_, 0)
        self.assertResultIsBOOL(AppKit.NSTextView.drawsBackground)
        self.assertArgIsBOOL(AppKit.NSTextView.setDrawsBackground_, 0)
        self.assertResultIsBOOL(AppKit.NSTextView.isFieldEditor)
        self.assertArgIsBOOL(AppKit.NSTextView.setFieldEditor_, 0)
        self.assertResultIsBOOL(AppKit.NSTextView.usesFontPanel)
        self.assertArgIsBOOL(AppKit.NSTextView.setUsesFontPanel_, 0)
        self.assertResultIsBOOL(AppKit.NSTextView.isRulerVisible)
        self.assertArgIsBOOL(AppKit.NSTextView.setRulerVisible_, 0)
        self.assertResultIsBOOL(AppKit.NSTextView.smartInsertDeleteEnabled)
        self.assertArgIsBOOL(AppKit.NSTextView.setSmartInsertDeleteEnabled_, 0)

        self.assertArgIsOut(AppKit.NSTextView.dragImageForSelectionWithEvent_origin_, 1)
        self.assertArgIsOut(
            AppKit.NSTextView.smartInsertForString_replacingRange_beforeString_afterString_,
            2,
        )
        self.assertArgIsOut(
            AppKit.NSTextView.smartInsertForString_replacingRange_beforeString_afterString_,
            3,
        )

    @min_os_level("10.10")
    def testMethods10_10(self):
        self.assertResultIsBOOL(AppKit.NSTextView.usesRolloverButtonForSelection)
        self.assertArgIsBOOL(AppKit.NSTextView.setUsesRolloverButtonForSelection_, 0)

    @min_os_level("10.7")
    def testMethods10_7(self):

        self.assertResultIsBOOL(AppKit.NSTextView.usesInspectorBar)
        self.assertArgIsBOOL(AppKit.NSTextView.setUsesInspectorBar_, 0)
        self.assertResultIsBOOL(AppKit.NSTextView.usesFindBar)
        self.assertArgIsBOOL(AppKit.NSTextView.setUsesFindBar_, 0)
        self.assertResultIsBOOL(AppKit.NSTextView.isIncrementalSearchingEnabled)
        self.assertArgIsBOOL(AppKit.NSTextView.setIncrementalSearchingEnabled_, 0)

    @min_os_level("10.5")
    def testMethods10_5(self):
        self.assertResultIsBOOL(AppKit.NSTextView.displaysLinkToolTips)
        self.assertArgIsBOOL(AppKit.NSTextView.setDisplaysLinkToolTips_, 0)
        self.assertResultIsBOOL(AppKit.NSTextView.isAutomaticQuoteSubstitutionEnabled)
        self.assertArgIsBOOL(AppKit.NSTextView.setAutomaticQuoteSubstitutionEnabled_, 0)
        self.assertResultIsBOOL(AppKit.NSTextView.isAutomaticLinkDetectionEnabled)
        self.assertArgIsBOOL(AppKit.NSTextView.setAutomaticLinkDetectionEnabled_, 0)

    @min_sdk_level("10.6")
    def testProtocolObjects(self):
        objc.protocolNamed("NSTextViewDelegate")

    def testProtocols(self):
        self.assertResultIsBOOL(TestNSTextViewHelper.textView_clickedOnLink_atIndex_)
        self.assertArgHasType(
            TestNSTextViewHelper.textView_clickedOnLink_atIndex_, 2, objc._C_NSUInteger
        )
        self.assertArgHasType(
            TestNSTextViewHelper.textView_clickedOnCell_inRect_atIndex_,
            2,
            AppKit.NSRect.__typestr__,
        )
        self.assertArgHasType(
            TestNSTextViewHelper.textView_clickedOnCell_inRect_atIndex_,
            3,
            objc._C_NSUInteger,
        )
        self.assertArgHasType(
            TestNSTextViewHelper.textView_doubleClickedOnCell_inRect_atIndex_,
            2,
            AppKit.NSRect.__typestr__,
        )
        self.assertArgHasType(
            TestNSTextViewHelper.textView_doubleClickedOnCell_inRect_atIndex_,
            3,
            objc._C_NSUInteger,
        )
        self.assertArgHasType(
            TestNSTextViewHelper.textView_draggedCell_inRect_event_atIndex_,
            2,
            AppKit.NSRect.__typestr__,
        )
        self.assertArgHasType(
            TestNSTextViewHelper.textView_draggedCell_inRect_event_atIndex_,
            4,
            objc._C_NSUInteger,
        )
        self.assertArgHasType(
            TestNSTextViewHelper.textView_writablePasteboardTypesForCell_atIndex_,
            2,
            objc._C_NSUInteger,
        )
        self.assertResultIsBOOL(
            TestNSTextViewHelper.textView_writeCell_atIndex_toPasteboard_type_
        )
        self.assertArgHasType(
            TestNSTextViewHelper.textView_writeCell_atIndex_toPasteboard_type_,
            2,
            objc._C_NSUInteger,
        )
        self.assertArgHasType(
            TestNSTextViewHelper.textView_willChangeSelectionFromCharacterRange_toCharacterRange_,  # noqa: B950
            1,
            AppKit.NSRange.__typestr__,
        )
        self.assertArgHasType(
            TestNSTextViewHelper.textView_willChangeSelectionFromCharacterRange_toCharacterRange_,  # noqa: B950
            2,
            AppKit.NSRange.__typestr__,
        )
        self.assertResultIsBOOL(
            TestNSTextViewHelper.textView_shouldChangeTextInRanges_replacementStrings_
        )
        self.assertArgHasType(
            TestNSTextViewHelper.textView_willDisplayToolTip_forCharacterAtIndex_,
            2,
            objc._C_NSUInteger,
        )
        self.assertArgHasType(
            TestNSTextViewHelper.textView_completions_forPartialWordRange_indexOfSelectedItem_,
            2,
            AppKit.NSRange.__typestr__,
        )
        self.assertArgHasType(
            TestNSTextViewHelper.textView_completions_forPartialWordRange_indexOfSelectedItem_,
            3,
            b"N^" + objc._C_NSInteger,
        )
        self.assertResultIsBOOL(
            TestNSTextViewHelper.textView_shouldChangeTextInRange_replacementString_
        )
        self.assertArgHasType(
            TestNSTextViewHelper.textView_shouldChangeTextInRange_replacementString_,
            1,
            AppKit.NSRange.__typestr__,
        )
        self.assertResultIsBOOL(TestNSTextViewHelper.textView_doCommandBySelector_)
        self.assertArgHasType(
            TestNSTextViewHelper.textView_doCommandBySelector_, 1, objc._C_SEL
        )
        self.assertResultIsBOOL(TestNSTextViewHelper.textView_clickedOnLink_)
        self.assertArgHasType(
            TestNSTextViewHelper.textView_clickedOnCell_inRect_,
            2,
            AppKit.NSRect.__typestr__,
        )
        self.assertArgHasType(
            TestNSTextViewHelper.textView_doubleClickedOnCell_inRect_,
            2,
            AppKit.NSRect.__typestr__,
        )
        self.assertArgHasType(
            TestNSTextViewHelper.textView_draggedCell_inRect_event_,
            2,
            AppKit.NSRect.__typestr__,
        )

    @min_os_level("10.5")
    def testProtocols10_5(self):
        self.assertArgHasType(
            TestNSTextViewHelper.textView_shouldSetSpellingState_range_,
            1,
            objc._C_NSInteger,
        )
        self.assertArgHasType(
            TestNSTextViewHelper.textView_shouldSetSpellingState_range_,
            2,
            AppKit.NSRange.__typestr__,
        )
        self.assertArgHasType(
            TestNSTextViewHelper.textView_menu_forEvent_atIndex_, 3, objc._C_NSUInteger
        )

    @min_os_level("10.6")
    def testMethods10_6(self):
        self.assertResultIsBOOL(AppKit.NSTextView.isCoalescingUndo)

        self.assertArgIsBOOL(AppKit.NSTextView.setAutomaticDataDetectionEnabled_, 0)
        self.assertResultIsBOOL(AppKit.NSTextView.isAutomaticDataDetectionEnabled)
        self.assertArgIsBOOL(AppKit.NSTextView.setAutomaticDashSubstitutionEnabled_, 0)
        self.assertResultIsBOOL(AppKit.NSTextView.isAutomaticDashSubstitutionEnabled)
        self.assertArgIsBOOL(AppKit.NSTextView.setAutomaticTextReplacementEnabled_, 0)
        self.assertResultIsBOOL(AppKit.NSTextView.isAutomaticTextReplacementEnabled)
        self.assertArgIsBOOL(
            AppKit.NSTextView.setAutomaticSpellingCorrectionEnabled_, 0
        )
        self.assertResultIsBOOL(AppKit.NSTextView.isAutomaticSpellingCorrectionEnabled)

        self.assertArgHasType(
            AppKit.NSTextView.checkTextInRange_types_options_,
            0,
            AppKit.NSRange.__typestr__,
        )
        self.assertArgHasType(
            AppKit.NSTextView.handleTextCheckingResults_forRange_types_options_orthography_wordCount_,  # noqa: B950
            1,
            AppKit.NSRange.__typestr__,
        )

    @min_os_level("10.12")
    def testMethods10_12(self):
        self.assertResultIsBOOL(AppKit.NSTextView.stronglyReferencesTextStorage)

        self.assertResultIsBOOL(AppKit.NSTextView.allowsCharacterPickerTouchBarItem)
        self.assertArgIsBOOL(AppKit.NSTextView.setAllowsCharacterPickerTouchBarItem_, 0)

    @min_os_level("10.14")
    def testMethods10_14(self):
        self.assertResultIsBOOL(
            AppKit.NSTextView.performValidatedReplacementInRange_withAttributedString_
        )
        self.assertResultIsBOOL(
            AppKit.NSTextView.usesAdaptiveColorMappingForDarkAppearance
        )
        self.assertArgIsBOOL(
            AppKit.NSTextView.setUsesAdaptiveColorMappingForDarkAppearance_, 0
        )

    @min_os_level("10.6")
    def testProtocols10_6(self):
        self.assertArgHasType(
            TestNSTextViewHelper.textView_willCheckTextInRange_options_types_,
            1,
            AppKit.NSRange.__typestr__,
        )
        self.assertArgHasType(
            TestNSTextViewHelper.textView_willCheckTextInRange_options_types_,
            3,
            b"N^" + objc._C_NSInteger,
        )

        self.assertArgHasType(
            TestNSTextViewHelper.textView_didCheckTextInRange_types_options_results_orthography_wordCount_,  # noqa: B950
            1,
            AppKit.NSRange.__typestr__,
        )
        self.assertArgHasType(
            TestNSTextViewHelper.textView_didCheckTextInRange_types_options_results_orthography_wordCount_,  # noqa: B950
            2,
            objc._C_NSInteger,
        )
        self.assertArgHasType(
            TestNSTextViewHelper.textView_didCheckTextInRange_types_options_results_orthography_wordCount_,  # noqa: B950
            6,
            objc._C_NSInteger,
        )

    @min_os_level("10.7")
    def testProtocols10_7(self):
        self.assertArgHasType(
            TestNSTextViewHelper.textView_URLForContentsOfTextAttachment_atIndex_,
            2,
            objc._C_NSUInteger,
        )

    @min_os_level("10.12")
    def testProtocols10_12(self):
        self.assertArgHasType(
            TestNSTextViewHelper.textView_candidatesForSelectedRange_,
            1,
            AppKit.NSRange.__typestr__,
        )
        self.assertArgHasType(
            TestNSTextViewHelper.textView_candidates_forSelectedRange_,
            2,
            AppKit.NSRange.__typestr__,
        )
        self.assertArgHasType(
            TestNSTextViewHelper.textView_shouldSelectCandidateAtIndex_,
            1,
            objc._C_NSUInteger,
        )
        self.assertResultIsBOOL(
            TestNSTextViewHelper.textView_shouldSelectCandidateAtIndex_
        )

    @min_os_level("10.14")
    def testProtocols10_14(self):
        objc.protocolNamed("NSUserActivityRestoring")



import sys
from math import floor

import Cocoa
import objc
from objc import super
from PyObjCTools import AppHelper


class Controller(Cocoa.NSObject):
    itsWindow = objc.IBOutlet()

    @objc.IBAction
    def changeTransparency_(self, sender):
        """."""
        self.itsWindow.setAlphaValue_(sender.floatValue())
        self.itsWindow.display()


class CustomView(Cocoa.NSView):
    circleImage = objc.ivar()
    pentaImage = objc.ivar()

    def awakeFromNib(self):
        self.circleImage = Cocoa.NSImage.imageNamed_("circle")
        if self.circleImage is None:
            sys.stderr.write("failed to access circle image\n")
            raise RuntimeError
        self.pentaImage = Cocoa.NSImage.imageNamed_("pentagram")
        if self.pentaImage is None:
            sys.stderr.write("failed to access pentagram image\n")
            raise RuntimeError
        self.setNeedsDisplay_(True)

    def drawRect_(self, rect):
        Cocoa.NSColor.clearColor().set()
        Cocoa.NSRectFill(self.frame())
        if self.window().alphaValue() > 0.7:
            self.circleImage.compositeToPoint_operation_(
                Cocoa.NSZeroPoint, Cocoa.NSCompositeSourceOver
            )
        else:
            self.pentaImage.compositeToPoint_operation_(
                Cocoa.NSZeroPoint, Cocoa.NSCompositeSourceOver
            )

        if floor(Cocoa.NSAppKitVersionNumber) <= Cocoa.NSAppKitVersionNumber10_1:
            self.window().setHasShadow_(False)
            self.window().setHasShadow_(True)
        else:
            self.window().invalidateShadow()


class CustomWindow(Cocoa.NSWindow):
    def awakeFromNib(self):
        self.initialLocation = Cocoa.NSPoint()

    def initWithContentRect_styleMask_backing_defer_(
        self, contentRect, aStyle, bufferingType, flag
    ):
        result = super().initWithContentRect_styleMask_backing_defer_(
            contentRect,
            Cocoa.NSBorderlessWindowMask,
            Cocoa.NSBackingStoreBuffered,
            False,
        )
        if result is None:
            sys.stderr.write("superclass call failed\n")
            raise RuntimeError
        result.setBackgroundColor_(Cocoa.NSColor.clearColor())
        result.setLevel_(Cocoa.NSStatusWindowLevel)
        result.setAlphaValue_(1.0)
        result.setOpaque_(False)
        result.setHasShadow_(True)
        return result

    def canBecomeKeyWindow(self):
        return True

    def mouseDragged_(self, theEvent):
        screenFrame = Cocoa.NSScreen.mainScreen().frame()
        if screenFrame is None:
            sys.stderr.write("failed to obtain screen\n")
            raise RuntimeError
        windowFrame = self.frame()
        if windowFrame is None:
            sys.stderr.write("failed to obtain frame\n")
            raise RuntimeError
        currentLocation = self.convertBaseToScreen_(
            self.mouseLocationOutsideOfEventStream()
        )
        newOrigin = Cocoa.NSMakePoint(
            (currentLocation.x - self.initialLocation.x),
            (currentLocation.y - self.initialLocation.y),
        )
        if (newOrigin.y + windowFrame.size.height) > (
            screenFrame.origin.y + screenFrame.size.height
        ):
            newOrigin.y = screenFrame.origin.y + (
                screenFrame.size.height + windowFrame.size.height
            )
        self.setFrameOrigin_(newOrigin)

    def mouseDown_(self, theEvent):
        windowFrame = self.frame()
        if windowFrame is None:
            sys.stderr.write("failed to obtain frame\n")
            raise RuntimeError
        self.initialLocation = self.convertBaseToScreen_(theEvent.locationInWindow())
        self.initialLocation.x -= windowFrame.origin.x
        self.initialLocation.y -= windowFrame.origin.y


if __name__ == "__main__":
    AppHelper.runEventLoop()



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSAccessibilityCustomAction(TestCase):
    @min_os_level("10.13")
    def testMethods(self):
        self.assertArgIsBlock(
            AppKit.NSAccessibilityCustomAction.initWithName_handler_, 1, b"Z"
        )
        self.assertArgIsSEL(
            AppKit.NSAccessibilityCustomAction.initWithName_target_selector_, 2, b"Z@:"
        )

        self.assertResultIsBlock(AppKit.NSAccessibilityCustomAction.handler, b"Z")
        self.assertArgIsBlock(AppKit.NSAccessibilityCustomAction.setHandler_, 0, b"Z")

        # XXX: Cannot describe this properly:
        # self.assertResultIsBlock(AppKit.NSAccessibilityCustomAction.selector, b'...')
        # self.assertArgIsBlock(AppKit.NSAccessibilityCustomAction.setSelector_, 0, b'...')



import AppKit
from PyObjCTools.TestSupport import TestCase


class TestNSCursor(TestCase):
    def testMethods(self):
        self.assertArgIsBOOL(AppKit.NSCursor.setHiddenUntilMouseMoves_, 0)
        self.assertArgIsBOOL(AppKit.NSCursor.setOnMouseExited_, 0)
        self.assertArgIsBOOL(AppKit.NSCursor.setOnMouseEntered_, 0)
        self.assertResultIsBOOL(AppKit.NSCursor.isSetOnMouseExited)
        self.assertResultIsBOOL(AppKit.NSCursor.isSetOnMouseEntered)

    def testConstants(self):
        self.assertEqual(AppKit.NSAppKitVersionNumberWithCursorSizeSupport, 682.0)



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level, min_sdk_level
import objc


class TestNSApplicationHelper(AppKit.NSObject):
    def applicationShouldAutomaticallyLocalizeKeyEquivalents_(self, a):
        pass

    def copyWithZone_(self, zone):
        return self

    def writeSelectionToPasteboard_types_(self, pb, tp):
        return 1

    def readSelectionFromPasteboard_(self, pb):
        return 1

    def application_openFile_(self, sender, file):
        return 1

    def application_openTempFile_(self, sender, file):
        return 1

    def applicationShouldOpenUntitledFile_(self, sender):
        return 1

    def application_openFileWithoutUI_(self, sender, file):
        return 1

    def application_printFile_(self, sender, file):
        return 1

    def application_printFiles_withSettings_showPrintPanels_(self, a, f, s, sh):
        return 1

    def applicationShouldTerminateAfterLastWindowClosed_(self, sender):
        return 1

    def applicationShouldHandleReopen_hasVisibleWindows_(self, sender, flag):
        return 1

    def application_willContinueUserActivityWithType_(self, sender, tp):
        return 1

    def application_continueUserActivity_restorationHandler_(self, sender, tp, h):
        return 1

    def application_delegateHandlesKey_(self, a, k):
        return 1

    def applicationSupportsSecureRestorableState_(self, a):
        return 1


class TestNSApplication(TestCase):
    def testConstants(self):
        self.assertIsInstance(AppKit.NSAppKitVersionNumber, float)

        self.assertEqual(AppKit.NSAppKitVersionNumber10_0, 577)
        self.assertEqual(AppKit.NSAppKitVersionNumber10_1, 620)
        self.assertEqual(AppKit.NSAppKitVersionNumber10_2, 663)
        self.assertEqual(AppKit.NSAppKitVersionNumber10_2_3, 663.6)
        self.assertEqual(AppKit.NSAppKitVersionNumber10_3, 743)
        self.assertEqual(AppKit.NSAppKitVersionNumber10_3_2, 743.14)
        self.assertEqual(AppKit.NSAppKitVersionNumber10_3_3, 743.2)
        self.assertEqual(AppKit.NSAppKitVersionNumber10_3_5, 743.24)
        self.assertEqual(AppKit.NSAppKitVersionNumber10_3_7, 743.33)
        self.assertEqual(AppKit.NSAppKitVersionNumber10_3_9, 743.36)
        self.assertEqual(AppKit.NSAppKitVersionNumber10_4, 824)
        self.assertEqual(AppKit.NSAppKitVersionNumber10_4_1, 824.1)
        self.assertEqual(AppKit.NSAppKitVersionNumber10_4_3, 824.23)
        self.assertEqual(AppKit.NSAppKitVersionNumber10_4_4, 824.33)
        self.assertEqual(AppKit.NSAppKitVersionNumber10_4_7, 824.41)
        self.assertEqual(AppKit.NSAppKitVersionNumber10_5, 949)
        self.assertEqual(AppKit.NSAppKitVersionNumber10_5_2, 949.27)
        self.assertEqual(AppKit.NSAppKitVersionNumber10_5_3, 949.33)
        self.assertEqual(AppKit.NSAppKitVersionNumber10_6, 1038)
        self.assertEqual(AppKit.NSAppKitVersionNumber10_7, 1138)
        self.assertEqual(AppKit.NSAppKitVersionNumber10_7_2, 1138.23)
        self.assertEqual(AppKit.NSAppKitVersionNumber10_7_3, 1138.32)
        self.assertEqual(AppKit.NSAppKitVersionNumber10_7_4, 1138.47)
        self.assertEqual(AppKit.NSAppKitVersionNumber10_8, 1187)
        self.assertEqual(AppKit.NSAppKitVersionNumber10_9, 1265)
        self.assertEqual(AppKit.NSAppKitVersionNumber10_10, 1343)
        self.assertEqual(AppKit.NSAppKitVersionNumber10_10_2, 1344)
        self.assertEqual(AppKit.NSAppKitVersionNumber10_10_3, 1347)
        self.assertEqual(AppKit.NSAppKitVersionNumber10_10_4, 1348)
        self.assertEqual(AppKit.NSAppKitVersionNumber10_10_5, 1348)
        self.assertEqual(AppKit.NSAppKitVersionNumber10_10_Max, 1349)
        self.assertEqual(AppKit.NSAppKitVersionNumber10_11, 1404)
        self.assertEqual(AppKit.NSAppKitVersionNumber10_11_1, 1404.13)
        self.assertEqual(AppKit.NSAppKitVersionNumber10_11_2, 1404.34)
        self.assertEqual(AppKit.NSAppKitVersionNumber10_11_3, 1404.34)
        self.assertEqual(AppKit.NSAppKitVersionNumber10_12, 1504)
        self.assertEqual(AppKit.NSAppKitVersionNumber10_12_1, 1504.60)
        self.assertEqual(AppKit.NSAppKitVersionNumber10_12_2, 1504.76)
        self.assertEqual(AppKit.NSAppKitVersionNumber10_13, 1561)
        self.assertEqual(AppKit.NSAppKitVersionNumber10_13_1, 1561.1)
        self.assertEqual(AppKit.NSAppKitVersionNumber10_13_2, 1561.2)
        self.assertEqual(AppKit.NSAppKitVersionNumber10_13_4, 1561.4)
        self.assertEqual(AppKit.NSAppKitVersionNumber10_14, 1671)
        self.assertEqual(AppKit.NSAppKitVersionNumber10_14_1, 1671.1)
        self.assertEqual(AppKit.NSAppKitVersionNumber10_14_2, 1671.2)
        self.assertEqual(AppKit.NSAppKitVersionNumber10_14_3, 1671.3)
        self.assertEqual(AppKit.NSAppKitVersionNumber10_14_4, 1671.4)
        self.assertEqual(AppKit.NSAppKitVersionNumber10_14_5, 1671.5)
        self.assertEqual(AppKit.NSAppKitVersionNumber10_15, 1894)
        self.assertEqual(AppKit.NSAppKitVersionNumber10_15_1, 1894.1)
        self.assertEqual(AppKit.NSAppKitVersionNumber10_15_2, 1894.2)
        self.assertEqual(AppKit.NSAppKitVersionNumber10_15_3, 1894.3)
        self.assertEqual(AppKit.NSAppKitVersionNumber10_15_4, 1894.4)
        self.assertEqual(AppKit.NSAppKitVersionNumber10_15_5, 1894.5)
        self.assertEqual(AppKit.NSAppKitVersionNumber10_15_6, 1894.6)
        self.assertEqual(AppKit.NSAppKitVersionNumber11_0, 2022)
        self.assertEqual(AppKit.NSAppKitVersionNumber11_1, 2022.2)
        self.assertEqual(AppKit.NSAppKitVersionNumber11_2, 2022.3)
        self.assertEqual(AppKit.NSAppKitVersionNumber11_3, 2022.4)
        self.assertEqual(AppKit.NSAppKitVersionNumber11_4, 2022.5)

        self.assertIsInstance(AppKit.NSModalPanelRunLoopMode, str)
        self.assertIsInstance(AppKit.NSEventTrackingRunLoopMode, str)

        self.assertEqual(AppKit.NSRunStoppedResponse, -1000)
        self.assertEqual(AppKit.NSRunAbortedResponse, -1001)
        self.assertEqual(AppKit.NSRunContinuesResponse, -1002)

        self.assertEqual(AppKit.NSUpdateWindowsRunLoopOrdering, 500_000)

        self.assertEqual(AppKit.NSCriticalRequest, 0)
        self.assertEqual(AppKit.NSInformationalRequest, 10)

        self.assertEqual(AppKit.NSApplicationDelegateReplySuccess, 0)
        self.assertEqual(AppKit.NSApplicationDelegateReplyCancel, 1)
        self.assertEqual(AppKit.NSApplicationDelegateReplyFailure, 2)

        self.assertEqual(AppKit.NSTerminateCancel, 0)
        self.assertEqual(AppKit.NSTerminateNow, 1)
        self.assertEqual(AppKit.NSTerminateLater, 2)

        self.assertEqual(AppKit.NSPrintingCancelled, 0)
        self.assertEqual(AppKit.NSPrintingSuccess, 1)
        self.assertEqual(AppKit.NSPrintingFailure, 3)
        self.assertEqual(AppKit.NSPrintingReplyLater, 2)

        self.assertIsInstance(AppKit.NSApplicationDidBecomeActiveNotification, str)
        self.assertIsInstance(AppKit.NSApplicationDidHideNotification, str)
        self.assertIsInstance(AppKit.NSApplicationDidFinishLaunchingNotification, str)
        self.assertIsInstance(AppKit.NSApplicationDidResignActiveNotification, str)
        self.assertIsInstance(AppKit.NSApplicationDidUnhideNotification, str)
        self.assertIsInstance(AppKit.NSApplicationDidUpdateNotification, str)
        self.assertIsInstance(AppKit.NSApplicationWillBecomeActiveNotification, str)
        self.assertIsInstance(AppKit.NSApplicationWillHideNotification, str)
        self.assertIsInstance(AppKit.NSApplicationWillFinishLaunchingNotification, str)
        self.assertIsInstance(AppKit.NSApplicationWillResignActiveNotification, str)
        self.assertIsInstance(AppKit.NSApplicationWillUnhideNotification, str)
        self.assertIsInstance(AppKit.NSApplicationWillUpdateNotification, str)
        self.assertIsInstance(AppKit.NSApplicationWillTerminateNotification, str)
        self.assertIsInstance(
            AppKit.NSApplicationDidChangeScreenParametersNotification, str
        )

        self.assertEqual(AppKit.NSWindowListOrderedFrontToBack, 1 << 0)

        self.assertEqual(AppKit.NSModalResponseStop, -1000)
        self.assertEqual(AppKit.NSModalResponseAbort, -1001)
        self.assertEqual(AppKit.NSModalResponseContinue, -1002)

    def testFunctions(self):
        # Testing the next function is not doable in this context...
        AppKit.NSApplicationMain
        self.assertResultIsBOOL(AppKit.NSApplicationLoad)
        self.assertIsInstance(AppKit.NSApplicationLoad(), bool)

        self.assertResultIsBOOL(AppKit.NSShowsServicesMenuItem)
        self.assertIsInstance(AppKit.NSShowsServicesMenuItem("foobar"), bool)

        self.assertIsInstance(AppKit.NSSetShowsServicesMenuItem("foobar", 1), int)
        self.assertArgIsBOOL(AppKit.NSSetShowsServicesMenuItem, 1)

        AppKit.NSUpdateDynamicServices()

        pboard = AppKit.NSPasteboard.pasteboardWithName_("pyobjctest")
        self.assertIsInstance(pboard, AppKit.NSPasteboard)
        self.assertIsInstance(AppKit.NSPerformService("help", pboard), bool)

        v = TestNSApplicationHelper.alloc().init()
        AppKit.NSRegisterServicesProvider(v, "foobar")
        AppKit.NSUnregisterServicesProvider("foobar")

    def testNSApp(self):
        self.assertIsNot(AppKit.NSApp, None)
        self.assertEqual(type(AppKit.NSApp).__name__, "_NSApp")
        self.assertHasAttr(AppKit.NSApp, "__call__")
        app = AppKit.NSApp()
        if app is not None:
            self.assertIsInstance(app, AppKit.NSApplication)

            self.assertIsInstance(AppKit.NSApp.description(), str)

            self.assertRaises(AttributeError, setattr, AppKit.NSApp, "foo", 42)

    def testNSModalSession(self):
        self.assertIsOpaquePointer(AppKit.NSModalSession)

        app = AppKit.NSApplication.sharedApplication()
        window = AppKit.NSWindow.alloc().init()
        session = app.beginModalSessionForWindow_(window)
        self.assertIsInstance(session, AppKit.NSModalSession)
        app.endModalSession_(session)

    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSApplication.isActive)
        self.assertResultIsBOOL(AppKit.NSApplication.isHidden)
        self.assertResultIsBOOL(AppKit.NSApplication.isRunning)
        self.assertArgIsBOOL(AppKit.NSApplication.activateIgnoringOtherApps_, 0)

        self.assertArgIsSEL(
            AppKit.NSApplication.beginSheet_modalForWindow_modalDelegate_didEndSelector_contextInfo_,  # noqa: B950
            3,
            b"v@:@" + objc._C_NSInteger + b"^v",
        )
        self.assertArgHasType(
            AppKit.NSApplication.beginSheet_modalForWindow_modalDelegate_didEndSelector_contextInfo_,  # noqa: B950
            4,
            b"^v",
        )

        self.assertArgIsBOOL(AppKit.NSApplication.postEvent_atStart_, 1)
        self.assertArgIsBOOL(AppKit.NSApplication.makeWindowsPerform_inOrder_, 1)
        self.assertArgIsBOOL(AppKit.NSApplication.setWindowsNeedUpdate_, 0)
        self.assertResultIsBOOL(AppKit.NSApplication.sendAction_to_from_)
        self.assertResultIsBOOL(AppKit.NSApplication.tryToPerform_with_)
        self.assertArgIsBOOL(AppKit.NSApplication.replyToApplicationShouldTerminate_, 0)
        self.assertArgIsBOOL(AppKit.NSApplication.addWindowsItem_title_filename_, 2)
        self.assertArgIsBOOL(AppKit.NSApplication.changeWindowsItem_title_filename_, 2)

        self.assertArgIsBOOL(
            AppKit.NSApplication.nextEventMatchingMask_untilDate_inMode_dequeue_, 3
        )

    def testDelegateMethods(self):
        self.assertResultIsBOOL(
            TestNSApplicationHelper.applicationShouldAutomaticallyLocalizeKeyEquivalents_
        )
        self.assertResultIsBOOL(TestNSApplicationHelper.application_openFile_)
        self.assertResultIsBOOL(TestNSApplicationHelper.application_openTempFile_)
        self.assertResultIsBOOL(
            TestNSApplicationHelper.applicationShouldOpenUntitledFile_
        )
        self.assertResultIsBOOL(TestNSApplicationHelper.application_openFileWithoutUI_)
        self.assertResultIsBOOL(TestNSApplicationHelper.application_printFile_)
        self.assertArgIsBOOL(
            TestNSApplicationHelper.application_printFiles_withSettings_showPrintPanels_,
            3,
        )
        self.assertResultIsBOOL(
            TestNSApplicationHelper.applicationShouldTerminateAfterLastWindowClosed_
        )
        self.assertResultIsBOOL(
            TestNSApplicationHelper.applicationShouldHandleReopen_hasVisibleWindows_
        )
        self.assertArgIsBOOL(
            TestNSApplicationHelper.applicationShouldHandleReopen_hasVisibleWindows_, 1
        )

        self.assertResultIsBOOL(
            TestNSApplicationHelper.writeSelectionToPasteboard_types_
        )
        self.assertResultIsBOOL(TestNSApplicationHelper.readSelectionFromPasteboard_)

        self.assertResultIsBOOL(
            TestNSApplicationHelper.application_willContinueUserActivityWithType_
        )
        self.assertResultIsBOOL(
            TestNSApplicationHelper.application_continueUserActivity_restorationHandler_
        )
        self.assertArgIsBlock(
            TestNSApplicationHelper.application_continueUserActivity_restorationHandler_,
            2,
            b"v@",
        )

        self.assertResultIsBOOL(TestNSApplicationHelper.application_delegateHandlesKey_)
        self.assertResultIsBOOL(
            TestNSApplicationHelper.applicationSupportsSecureRestorableState_
        )

    @min_os_level("10.6")
    def testConstants10_6(self):
        self.assertEqual(AppKit.NSAppKitVersionNumber10_4_1, 824.1)
        self.assertEqual(AppKit.NSAppKitVersionNumber10_4_3, 824.23)
        self.assertEqual(AppKit.NSAppKitVersionNumber10_4_4, 824.33)
        self.assertEqual(AppKit.NSAppKitVersionNumber10_4_7, 824.41)
        self.assertEqual(AppKit.NSAppKitVersionNumber10_5, 949)
        self.assertEqual(AppKit.NSAppKitVersionNumber10_5_2, 949.27)
        self.assertEqual(AppKit.NSAppKitVersionNumber10_5_3, 949.33)
        self.assertEqual(AppKit.NSAppKitVersionNumber10_5_3, 949.33)

        self.assertEqual(AppKit.NSApplicationPresentationDefault, 0)
        self.assertEqual(AppKit.NSApplicationPresentationAutoHideDock, (1 << 0))
        self.assertEqual(AppKit.NSApplicationPresentationHideDock, (1 << 1))
        self.assertEqual(AppKit.NSApplicationPresentationAutoHideMenuBar, (1 << 2))
        self.assertEqual(AppKit.NSApplicationPresentationHideMenuBar, (1 << 3))
        self.assertEqual(AppKit.NSApplicationPresentationDisableAppleMenu, (1 << 4))
        self.assertEqual(
            AppKit.NSApplicationPresentationDisableProcessSwitching, (1 << 5)
        )
        self.assertEqual(AppKit.NSApplicationPresentationDisableForceQuit, (1 << 6))
        self.assertEqual(
            AppKit.NSApplicationPresentationDisableSessionTermination, (1 << 7)
        )
        self.assertEqual(
            AppKit.NSApplicationPresentationDisableHideApplication, (1 << 8)
        )
        self.assertEqual(
            AppKit.NSApplicationPresentationDisableMenuBarTransparency, (1 << 9)
        )

        self.assertEqual(AppKit.NSUserInterfaceLayoutDirectionLeftToRight, 0)
        self.assertEqual(AppKit.NSUserInterfaceLayoutDirectionRightToLeft, 1)

    @min_os_level("10.7")
    def testConstants10_7(self):
        self.assertEqual(AppKit.NSApplicationPresentationFullScreen, 1 << 10)
        self.assertEqual(AppKit.NSApplicationPresentationAutoHideToolbar, 1 << 11)
        self.assertEqual(AppKit.NSRemoteNotificationTypeNone, 0)
        self.assertEqual(AppKit.NSRemoteNotificationTypeBadge, 1)

        self.assertEqual(AppKit.NSAppKitVersionNumber10_7, 1138)
        self.assertEqual(AppKit.NSAppKitVersionNumber10_7_2, 1138.23)

        self.assertIsInstance(AppKit.NSApplicationLaunchRemoteNotificationKey, str)
        self.assertIsInstance(AppKit.NSApplicationLaunchIsDefaultLaunchKey, str)

    @min_os_level("10.8")
    def testConstants10_8(self):
        self.assertIsInstance(AppKit.NSApplicationLaunchUserNotificationKey, str)
        self.assertIsInstance(AppKit.NSTextAlternativesAttributeName, str)
        self.assertIsInstance(AppKit.NSUsesScreenFontsDocumentAttribute, str)

        self.assertEqual(AppKit.NSRemoteNotificationTypeSound, 2)
        self.assertEqual(AppKit.NSRemoteNotificationTypeAlert, 4)

    @min_os_level("10.9")
    def testConstants10_9(self):
        self.assertEqual(AppKit.NSApplicationOcclusionStateVisible, 1 << 1)

        self.assertIsInstance(
            AppKit.NSApplicationDidChangeOcclusionStateNotification, str
        )

    @min_os_level("10.11.2")
    def testConstants10_11_2(self):
        self.assertEqual(
            AppKit.NSApplicationPresentationDisableCursorLocationAssistance, 1 << 12
        )

    @min_os_level("10.13")
    def testConstants10_13(self):
        self.assertIsInstance(AppKit.NSAboutPanelOptionCredits, str)
        self.assertIsInstance(AppKit.NSAboutPanelOptionApplicationName, str)
        self.assertIsInstance(AppKit.NSAboutPanelOptionApplicationIcon, str)
        self.assertIsInstance(AppKit.NSAboutPanelOptionVersion, str)
        self.assertIsInstance(AppKit.NSAboutPanelOptionApplicationVersion, str)

    @min_os_level("10.14")
    def testConstants10_14(self):
        self.assertIsInstance(AppKit.NSAppearanceDocumentAttribute, str)

    @min_os_level("12.0")
    def testConstants12_0(self):
        self.assertIsInstance(
            AppKit.NSApplicationProtectedDataWillBecomeUnavailableNotification, str
        )
        self.assertIsInstance(
            AppKit.NSApplicationProtectedDataDidBecomeAvailableNotification, str
        )

    @min_os_level("10.6")
    def testMethods10_6(self):
        self.assertResultIsBOOL(AppKit.NSApplication.setActivationPolicy_)
        self.assertResultIsBOOL(AppKit.NSApplication.isFullKeyboardAccessEnabled)

    @min_os_level("10.12")
    def testMethods10_12(self):
        self.assertArgIsBlock(
            AppKit.NSApplication.enumerateWindowsWithOptions_usingBlock_, 1, b"v@o^Z"
        )

    @min_os_level("10.14")
    def testMethods10_14(self):
        self.assertResultIsBOOL(AppKit.NSApplication.isRegisteredForRemoteNotifications)

    @min_os_level("12.0")
    def testMethods12_0(self):
        self.assertResultIsBOOL(AppKit.NSApplication.isProtectedDataAvailable)
        self.assertResultIsBOOL(AppKit.NSApplication.isProtectedDataAvailable)

    @min_sdk_level("10.10")
    def testProtocols(self):
        objc.protocolNamed("NSApplicationDelegate")



# This is not a real test, but more documentation of a strange feature of
# the Cocoa runtime.
#
# A number of classes in Cocoa grow new methods if you instantiate them. We
# work around this feature by rescanning the method table after calling a
# class-method.
#
# We need to do this to reliably detect calls to the superclass implementation
# of a method. Without the workaround, calls to NSButtonCell.isEnabled_ (one
# of the magical classes) would be interpreted as calls to
# NSActionCell.isEnabled_, which is wrong.
#

from PyObjCTools.TestSupport import TestCase, max_sdk_level
import objc

import AppKit  # noqa: F401


class TestWeirdness(TestCase):
    def doWeirdness(self, className, methodToTest):
        c = objc.lookUpClass(className)
        before = getattr(c, methodToTest)  # noqa: F841
        b = c.alloc().init()  # noqa: F841
        after = getattr(c, methodToTest)

        self.assertEqual(after.definingClass, c)

    @max_sdk_level("10.14")
    def testWeirdness1(self):
        self.doWeirdness("NSButtonCell", "setEnabled_")

    @max_sdk_level("10.14")
    def testWeirdness2(self):
        self.doWeirdness("NSTextView", "setEditable_")



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level, min_sdk_level
import objc


class TestNSSplitViewHelper(AppKit.NSObject):
    def splitView_canCollapseSubview_(self, sp, sv):
        return 1

    def splitView_shouldCollapseSubview_forDoubleClickOnDividerAtIndex_(
        self, sp, sv, i
    ):
        return 1

    def splitView_constrainMinCoordinate_ofSubviewAt_(self, sv, c, i):
        return 1

    def splitView_constrainMaxCoordinate_ofSubviewAt_(self, sv, c, i):
        return 1

    def splitView_constrainSplitPosition_ofSubviewAt_(self, sv, c, i):
        return 1

    def splitView_resizeSubviewsWithOldSize_(self, sv, sz):
        pass

    def splitView_shouldHideDividerAtIndex_(self, sv, i):
        return 1

    def splitView_effectiveRect_forDrawnRect_ofDividerAtIndex_(self, sv, er, dr, i):
        return 1

    def splitView_additionalEffectiveRectOfDividerAtIndex_(self, sv, i):
        return 1

    def splitViewWillResizeSubviews_(self, nt):
        pass

    def splitViewDidResizeSubviews_(self, nt):
        pass

    def splitView_shouldAdjustSizeOfSubview_(self, s, sv):
        return 1


class TestNSSplitView(TestCase):
    def testConstants(self):
        self.assertEqual(AppKit.NSSplitViewDividerStyleThick, 1)
        self.assertEqual(AppKit.NSSplitViewDividerStyleThin, 2)

        self.assertIsInstance(AppKit.NSSplitViewWillResizeSubviewsNotification, str)
        self.assertIsInstance(AppKit.NSSplitViewDidResizeSubviewsNotification, str)

    @min_os_level("10.6")
    def testConstants10_6(self):
        self.assertEqual(AppKit.NSSplitViewDividerStylePaneSplitter, 3)

    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSSplitView.isVertical)
        self.assertArgIsBOOL(AppKit.NSSplitView.setVertical_, 0)
        self.assertResultIsBOOL(AppKit.NSSplitView.isSubviewCollapsed_)
        self.assertResultIsBOOL(AppKit.NSSplitView.isPaneSplitter)
        self.assertArgIsBOOL(AppKit.NSSplitView.setIsPaneSplitter_, 0)

    @min_os_level("10.11")
    def testMethods10_11(self):
        self.assertResultIsBOOL(AppKit.NSSplitView.arrangesAllSubviews)
        self.assertArgIsBOOL(AppKit.NSSplitView.setArrangesAllSubviews_, 0)

    @min_sdk_level("10.7")
    def testProtocolObjects(self):
        objc.protocolNamed("NSSplitViewDelegate")

    def testProtocol(self):
        self.assertResultIsBOOL(TestNSSplitViewHelper.splitView_canCollapseSubview_)
        self.assertResultIsBOOL(
            TestNSSplitViewHelper.splitView_shouldCollapseSubview_forDoubleClickOnDividerAtIndex_  # noqa: B950
        )
        self.assertArgHasType(
            TestNSSplitViewHelper.splitView_shouldCollapseSubview_forDoubleClickOnDividerAtIndex_,  # noqa: B950
            2,
            objc._C_NSInteger,
        )
        self.assertResultHasType(
            TestNSSplitViewHelper.splitView_constrainMinCoordinate_ofSubviewAt_,
            objc._C_CGFloat,
        )
        self.assertArgHasType(
            TestNSSplitViewHelper.splitView_constrainMinCoordinate_ofSubviewAt_,
            1,
            objc._C_CGFloat,
        )
        self.assertArgHasType(
            TestNSSplitViewHelper.splitView_constrainMinCoordinate_ofSubviewAt_,
            2,
            objc._C_NSInteger,
        )
        self.assertResultHasType(
            TestNSSplitViewHelper.splitView_constrainMaxCoordinate_ofSubviewAt_,
            objc._C_CGFloat,
        )
        self.assertArgHasType(
            TestNSSplitViewHelper.splitView_constrainMaxCoordinate_ofSubviewAt_,
            1,
            objc._C_CGFloat,
        )
        self.assertArgHasType(
            TestNSSplitViewHelper.splitView_constrainMaxCoordinate_ofSubviewAt_,
            2,
            objc._C_NSInteger,
        )
        self.assertResultHasType(
            TestNSSplitViewHelper.splitView_constrainSplitPosition_ofSubviewAt_,
            objc._C_CGFloat,
        )
        self.assertArgHasType(
            TestNSSplitViewHelper.splitView_constrainSplitPosition_ofSubviewAt_,
            1,
            objc._C_CGFloat,
        )
        self.assertArgHasType(
            TestNSSplitViewHelper.splitView_constrainSplitPosition_ofSubviewAt_,
            2,
            objc._C_NSInteger,
        )
        self.assertArgHasType(
            TestNSSplitViewHelper.splitView_resizeSubviewsWithOldSize_,
            1,
            AppKit.NSSize.__typestr__,
        )

    @min_os_level("10.5")
    def testProtocol10_5(self):
        self.assertResultIsBOOL(
            TestNSSplitViewHelper.splitView_shouldHideDividerAtIndex_
        )
        self.assertArgHasType(
            TestNSSplitViewHelper.splitView_shouldHideDividerAtIndex_,
            1,
            objc._C_NSInteger,
        )

        self.assertResultHasType(
            TestNSSplitViewHelper.splitView_effectiveRect_forDrawnRect_ofDividerAtIndex_,
            AppKit.NSRect.__typestr__,
        )
        self.assertArgHasType(
            TestNSSplitViewHelper.splitView_effectiveRect_forDrawnRect_ofDividerAtIndex_,
            1,
            AppKit.NSRect.__typestr__,
        )
        self.assertArgHasType(
            TestNSSplitViewHelper.splitView_effectiveRect_forDrawnRect_ofDividerAtIndex_,
            2,
            AppKit.NSRect.__typestr__,
        )
        self.assertArgHasType(
            TestNSSplitViewHelper.splitView_effectiveRect_forDrawnRect_ofDividerAtIndex_,
            3,
            objc._C_NSInteger,
        )

        self.assertResultHasType(
            TestNSSplitViewHelper.splitView_additionalEffectiveRectOfDividerAtIndex_,
            AppKit.NSRect.__typestr__,
        )
        self.assertArgHasType(
            TestNSSplitViewHelper.splitView_additionalEffectiveRectOfDividerAtIndex_,
            1,
            objc._C_NSInteger,
        )

    @min_os_level("10.6")
    def testProtocol10_6(self):
        self.assertResultIsBOOL(
            TestNSSplitViewHelper.splitView_shouldAdjustSizeOfSubview_
        )



import os
import sys

import CoreFoundation
from PyObjCTools.TestSupport import TestCase


class TestURLAccess(TestCase):
    def testConstants(self):
        self.assertEqual(CoreFoundation.kCFURLUnknownError, -10)
        self.assertEqual(CoreFoundation.kCFURLUnknownSchemeError, -11)
        self.assertEqual(CoreFoundation.kCFURLResourceNotFoundError, -12)
        self.assertEqual(CoreFoundation.kCFURLResourceAccessViolationError, -13)
        self.assertEqual(CoreFoundation.kCFURLRemoteHostUnavailableError, -14)
        self.assertEqual(CoreFoundation.kCFURLImproperArgumentsError, -15)
        self.assertEqual(CoreFoundation.kCFURLUnknownPropertyKeyError, -16)
        self.assertEqual(CoreFoundation.kCFURLPropertyKeyUnavailableError, -17)
        self.assertEqual(CoreFoundation.kCFURLTimeoutError, -18)
        self.assertIsInstance(CoreFoundation.kCFURLFileExists, str)
        self.assertIsInstance(CoreFoundation.kCFURLFileDirectoryContents, str)
        self.assertIsInstance(CoreFoundation.kCFURLFileLength, str)
        self.assertIsInstance(CoreFoundation.kCFURLFileLastModificationTime, str)
        self.assertIsInstance(CoreFoundation.kCFURLFilePOSIXMode, str)
        self.assertIsInstance(CoreFoundation.kCFURLFileOwnerID, str)
        self.assertIsInstance(CoreFoundation.kCFURLHTTPStatusCode, str)
        self.assertIsInstance(CoreFoundation.kCFURLHTTPStatusLine, str)

    def testFunctions(self):
        url = CoreFoundation.CFURLCreateWithFileSystemPath(
            None, __file__, CoreFoundation.kCFURLPOSIXPathStyle, False
        )

        self.assertArgIsOut(CoreFoundation.CFURLCreatePropertyFromResource, 3)
        val, errorCode = CoreFoundation.CFURLCreatePropertyFromResource(
            None, url, CoreFoundation.kCFURLFileExists, None
        )
        self.assertIsInstance(errorCode, int)
        self.assertIs(val, True)
        self.assertResultIsBOOL(CoreFoundation.CFURLCreateDataAndPropertiesFromResource)
        self.assertArgIsOut(CoreFoundation.CFURLCreateDataAndPropertiesFromResource, 2)
        self.assertArgIsOut(CoreFoundation.CFURLCreateDataAndPropertiesFromResource, 3)
        self.assertArgIsOut(CoreFoundation.CFURLCreateDataAndPropertiesFromResource, 5)
        (
            ok,
            data,
            properties,
            errorCode,
        ) = CoreFoundation.CFURLCreateDataAndPropertiesFromResource(  # noqa: B950
            None, url, None, None, None, None
        )
        self.assertTrue(ok)
        self.assertIsInstance(data, CoreFoundation.CFDataRef)
        self.assertIsInstance(properties, CoreFoundation.CFDictionaryRef)
        self.assertIsInstance(errorCode, int)
        self.assertTrue(properties[CoreFoundation.kCFURLFileExists])

        self.assertResultIsBOOL(CoreFoundation.CFURLWriteDataAndPropertiesToResource)
        self.assertArgIsOut(CoreFoundation.CFURLWriteDataAndPropertiesToResource, 3)
        url = CoreFoundation.CFURLCreateWithFileSystemPath(
            None, __file__ + "TEST", CoreFoundation.kCFURLPOSIXPathStyle, False
        )

        if sys.version_info[0] == 3:

            def buffer(value):
                return value

        else:
            from __builtin__ import buffer

        ok, errorCode = CoreFoundation.CFURLWriteDataAndPropertiesToResource(
            url, buffer(b"foobar"), None, None
        )
        self.assertTrue(ok)
        self.assertIsInstance(errorCode, int)
        self.assertTrue(os.path.exists(__file__ + "TEST"))
        with open(__file__ + "TEST") as fp:
            data = fp.read()
        self.assertEqual(data, "foobar")
        self.assertResultIsBOOL(CoreFoundation.CFURLDestroyResource)
        self.assertArgIsOut(CoreFoundation.CFURLDestroyResource, 1)
        ok, errorCode = CoreFoundation.CFURLDestroyResource(url, None)
        self.assertTrue(ok)
        self.assertIsInstance(errorCode, int)
        self.assertFalse(os.path.exists(__file__ + "TEST"))



import AppKit
from PyObjCTools.TestSupport import TestCase


class TestNSInterfaceStyle(TestCase):
    def testConstants(self):
        self.assertEqual(AppKit.NSNoInterfaceStyle, 0)
        self.assertEqual(AppKit.NSNextStepInterfaceStyle, 1)
        self.assertEqual(AppKit.NSWindows95InterfaceStyle, 2)
        self.assertEqual(AppKit.NSMacintoshInterfaceStyle, 3)

        self.assertIsInstance(AppKit.NSInterfaceStyleDefault, str)

    def testFunctions(self):
        v = AppKit.NSInterfaceStyleForKey("button", None)
        self.assertIsInstance(v, int)



"""
Some tests for difficult wrapped graphics functions and methods

NOTE: These require a WindowServer connection on macOS
"""
import array
import unittest

import Cocoa
from PyObjCTools.TestSupport import TestCase


class SimpleImage:
    """
    Helper class that makes it easier to access individual pixels in
    a bitmap image
    """

    def __init__(self, image):
        data = image.TIFFRepresentation()
        bitmap = Cocoa.NSBitmapImageRep.imageRepWithData_(data)

        self.bitmap = bitmap
        self.data = bitmap.bitmapData()
        self.rowbytes = bitmap.bytesPerRow()
        self.pixbytes = bitmap.bitsPerPixel() // 8
        self.rowCount = bitmap.pixelsHigh()

        if bitmap.isPlanar():
            raise ValueError("Planar image!")
        if (bitmap.bitsPerPixel() % 8) != 0:
            raise ValueError("bits per pixel isn't multiple of 8")

    def width(self):
        return self.bitmap.pixelsWide()

    def height(self):
        return self.bitmap.pixelsHigh()

    def getPixel(self, x, y):
        x = self.rowCount - 1 - x
        rowOffset = x * self.rowbytes
        pixelOffset = y * self.pixbytes
        offset = rowOffset + pixelOffset

        pixel = self.data[offset : offset + self.pixbytes]
        return pixel


class RectTest(TestCase):
    def setUp(self):

        # Force NSApp initialisation, needed for some of the tests
        Cocoa.NSApplication.sharedApplication().activateIgnoringOtherApps_(0)

        self.points = (
            ((10, 0), (1, 1)),
            ((10, 10), (1, 1)),
            ((0, 10), (1, 1)),
            ((0, 0), (1, 1)),
            ((70, 70), (10, 10)),
        )

        self.image = Cocoa.NSImage.alloc().initWithSize_((100, 100))

    def makeArray(self, points):
        code = "d"

        a = array.array(code, len(points) * [0, 0, 0, 0])
        for i in range(len(points)):
            p = points[i]
            a[(i * 4) + 0] = p[0][0]
            a[(i * 4) + 1] = p[0][1]
            a[(i * 4) + 2] = p[1][0]
            a[(i * 4) + 3] = p[1][1]

        return a

    def assertImagePoints(self, image, points):
        """
        Check that the image contains white pixels everywhere but at the
        specified locations points is sequence of NSRect values.
        """

        img = SimpleImage(image)

        if img.bitmap.pixelsWide() != img.bitmap.size()[0]:
            raise unittest.SkipTest("Test doesn't work with retina")

        allpoints = [(x, y) for x in range(img.width()) for y in range(img.height())]

        # Check black points
        def getPixel(img, x, y):
            value = img.getPixel(x, y)
            return bytes(value)

        for ((x, y), (h, w)) in points:
            for ox in range(w):
                for oy in range(h):
                    allpoints.remove((x + ox, y + oy))
                    self.assertEqual(
                        getPixel(img, x + ox, y + oy),
                        b"\x00\x00\x00\xff",
                        "Black pixel at %d,%d" % (x + ox, y + oy),
                    )

        # And white points
        for x, y in allpoints:
            self.assertEqual(
                getPixel(img, x, y),
                b"\x00\x00\x00\x00",
                "White pixel at %d,%d" % (x, y),
            )

    def tearDown(self):
        pass

    def test_NSRectFillList_tuple(self):
        """
        Check NSRectFillList with a tuple of NSRects
        """
        self.image.lockFocus()
        Cocoa.NSRectFillList(self.points, len(self.points))
        self.image.unlockFocus()

        self.assertImagePoints(self.image, self.points)

    def test_NSRectFillList_array(self):
        """
        Check NSRectFillList with a array.array of NSRects
        """
        self.image.lockFocus()
        Cocoa.NSRectFillList(self.makeArray(self.points), len(self.points))
        self.image.unlockFocus()
